
from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.generate_statement_body.begin_keyword)


class rule_006(token_indent):
    '''
    This rule checks the indent of the **begin** keyword.

    **Violation**

    .. code-block:: vhdl

       ram_array : for i in 0 to 7 generate
          begin

    **Fix**

    .. code-block:: vhdl

       ram_array : for i in 0 to 7 generate
       begin
    '''

    def __init__(self):
        token_indent.__init__(self, 'generate', '006', lTokens)




from vsg.vhdlFile.extract import tokens


def get_n_tokens_before_and_after_tokens(iToken, lTokens, lAllTokens, oTokenMap):
    lReturn = []
    lIndexes = []

    for oToken in lTokens:
        lIndexes.extend(oTokenMap.get_token_indexes(oToken))

    lIndexes.sort()

    for iIndex in lIndexes:
        iLine = oTokenMap.get_line_number_of_index(iIndex)
        lReturn.append(tokens.New(iIndex - iToken, iLine, lAllTokens[iIndex - iToken:iIndex + iToken + 1]))

    return lReturn




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.process_statement.end_process_label)


class rule_019(token_case_with_prefix_suffix):
    '''
    This rule checks the **end process** label has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       end process PROC_A;

    **Fix**

    .. code-block:: vhdl

       end process proc_a;
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'process', '019', lTokens)
        self.groups.append('case::label')




from vsg.token import procedure_call as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import actual_parameter_part


lExceptions = ['<=', 'end', 'map', 'component', 'entity', 'configuration', 'if']


def detect(iToken, lObjects):
    '''
    Calling functions:

    concurrent_procedure_call_statement ::=
        [ label : ] [ postponed ] procedure_call ;

    procedure_call_statement ::=
        [ label : ] procedure_call ;

    --------------------------

    procedure_call ::=
        *procedure*_name [ ( actual_parameter_part ) ]

    Differentiating a procedure call from anything else is essentially the absence of keywords.
    '''

    iCurrent = iToken

    while lObjects[iCurrent].get_value() != ';':
        if utils.is_item(lObjects, iCurrent):
            if lObjects[iCurrent].get_value().lower() in lExceptions:
                return False
        iCurrent += 1

    return True


def classify(iToken, lObjects):
    '''
    procedure_call ::=
        *procedure*_name [ ( actual_parameter_part ) ]
    '''

    iCurrent = utils.assign_next_token(token.procedure_name, iToken, lObjects)

    if utils.is_next_token('(', iToken, lObjects):
        iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iCurrent, lObjects)

        iCurrent = actual_parameter_part.classify(iCurrent, lObjects)

        iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.return_statement.return_keyword)


class rule_016(token_indent):
    '''
    This rule checks the indent of return statements in function bodies.

    **Violation**

    .. code-block:: vhdl

       function func1 return integer is
       begin
            return 99;
       return 99;
       end func1;

    **Fix**

    .. code-block:: vhdl

       function func1 return integer is
       begin
         return 99;
         return 99;
       end func1;
    '''

    def __init__(self):
        token_indent.__init__(self, 'function', '016', lTokens)




from vsg import parser

from vsg.token import range_constraint as token

from vsg.vhdlFile import utils


def detect(iToken, lObjects):
    '''
    range_constraint ::=
        **range** range
    '''
    if utils.is_next_token('range', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):
    iCurrent = utils.assign_next_token_required('range', token.range_keyword, iToken, lObjects)

    iParenCnt = 0
    while not utils.is_next_token_one_of([';', 'units', ':='], iCurrent, lObjects):
        iCurrent = utils.find_next_token(iCurrent, lObjects)
        iParenCnt = utils.update_paren_counter(iCurrent, lObjects, iParenCnt)
        if iParenCnt == -1:
            break
        iCurrent = utils.assign_next_token(parser.todo, iCurrent, lObjects)

    return iCurrent




from vsg import parser


class semicolon(parser.semicolon):
    '''
    unique_id = subprogram_declaration : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg import parser


class keyword(parser.keyword):
    '''
    unique_id = library_clause : keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = library_clause : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg import violation

from vsg.token import port_clause as token
from vsg.token import mode
from vsg.rule_group import structure


class rule_023(structure.Rule):
    '''
    This rule checks for missing modes in port declarations.

    .. NOTE:: This must be fixed by the user.  VSG makes no assumption on the direction of the port.

    **Violation**

    .. code-block:: vhdl

       port (
         WR_EN    : std_logic;
         RD_EN    : std_logic;
         OVERFLOW : std_logic;
         DATA     : inout std_logic
       );

    **Fix**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;
         DATA     : inout std_logic
       );
    '''

    def __init__(self):
        structure.Rule.__init__(self, name='port', identifier='023')
        self.solution = 'Add mode'
        self.fixable = False
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_interface_elements_between_tokens(token.open_parenthesis, token.close_parenthesis)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            for oToken in lTokens:
                if isinstance(oToken, mode.mode):
                    break
            else:
                self.add_violation(violation.New(oToi.get_line_number(), oToi, self.solution))

    def _fix_violation(self, oViolation):
        '''
        Applies fixes for any rule violations.
        '''
        return None




from vsg.rules.whitespace_after_token import Rule

from vsg.token import iteration_scheme as token

lTokens = []
lTokens.append(token.while_keyword)


class rule_100(Rule):
    '''
    This rule checks that a single space exists after the **while** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       while(condition)

       while      (condition)

    **Fix**

    .. code-block:: vhdl

       while (condition)

       while (condition)
    '''
    def __init__(self):
        Rule.__init__(self, 'iteration_scheme', '100', lTokens)




from vsg import parser


class identifier(parser.identifier):
    '''
    unique_id = identifier : identifier
    '''

    def __init__(self, sString):
        parser.identifier.__init__(self, sString)


class todo(parser.identifier):
    '''
    unique_id = identifier : todo
    '''

    def __init__(self, sString):
        parser.identifier.__init__(self, sString)




from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.full_type_declaration.identifier)
lAlign.append(token.incomplete_type_declaration.identifier)
lAlign.append(token.file_declaration.identifier)
lAlign.append(token.constant_declaration.identifier)
lAlign.append(token.signal_declaration.identifier)
lAlign.append(token.subtype_declaration.identifier)
lAlign.append(token.variable_declaration.identifier)
lAlign.append(token.alias_declaration.alias_designator)

lUnless = []
lUnless.append([token.subprogram_body.is_keyword,token.subprogram_body.begin_keyword])


class rule_400(align_tokens_in_region_between_tokens_unless_between_tokens):
    '''
    This rule checks the identifiers for all declarations are aligned in the block declarative region.

    |configuring_identifier_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       variable    var1 : natural;
       constant  c_period : time;

    **Fix**

    .. code-block:: vhdl

       variable var1 : natural;
       constant c_period : time;
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_unless_between_tokens.__init__(self, 'block', '400', lAlign, token.block_statement.block_keyword, token.block_statement.begin_keyword, lUnless)
        self.solution = 'Align identifer.'
        self.configuration_documentation_link = 'configuring_identifier_alignment_rules_link'




from vsg.rules import token_indent

from vsg.token import architecture_body as token


class rule_008(token_indent):
    '''
    This rule checks for spaces before the **end architecture** keywords.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of fifo is
       begin
         end architecture

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is
       begin
       end architecture
    '''

    def __init__(self):
        token_indent.__init__(self, 'architecture', '008', [token.end_keyword])




from vsg.rules.whitespace_after_token import Rule

from vsg.token import alias_declaration as token


class rule_100(Rule):
    '''
    This rule checks for a single space after the colon for the subtype_indication.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       alias alias_designator :    subtype_indication is name;
       alias alias_designator :subtype_indication is name;

    **Fix**

    .. code-block:: vhdl

       alias alias_designator : subtype_indication is name;
       alias alias_designator : subtype_indication is name;
    '''
    def __init__(self):
        Rule.__init__(self, 'alias_declaration', '100', [token.colon])




from vsg.token import file_type_definition as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import type_mark


def detect(iToken, lObjects):
    '''
    file_type_definition ::=
        file of type_mark
    '''

    if utils.is_next_token('file', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('file', token.file_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('of', token.of_keyword, iCurrent, lObjects)

    iCurrent = type_mark.classify(iToken, lObjects)

    return iCurrent




from vsg import parser


class protected_keyword(parser.keyword):
    '''
    unique_id = protected_type_body : protected_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class body_keyword(parser.keyword):
    '''
    unique_id = protected_type_body : body_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_keyword(parser.keyword):
    '''
    unique_id = protected_type_body : end_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_protected_keyword(parser.keyword):
    '''
    unique_id = protected_type_body : end_protected_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_body_keyword(parser.keyword):
    '''
    unique_id = protected_type_body : end_body_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class protected_type_simple_name(parser.simple_name):
    '''
    unique_id = protected_type_body : protected_type_simple_name
    '''

    def __init__(self, sString):
        parser.simple_name.__init__(self, sString)




from vsg.token import signal_kind as token

from vsg.vhdlFile import utils


def detect(iToken, lObjects):
    '''
    signal_kind ::=
        register | bus
    '''

    if utils.is_next_token('register', iToken, lObjects):
        return classify(iToken, lObjects)
    elif utils.is_next_token('bus', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_if('register', token.register_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_if('bus', token.register_bus, iToken, lObjects)

    return iCurrent




from vsg.vhdlFile.extract import utils

from vsg.vhdlFile.extract import tokens

from vsg.vhdlFile import utils as vhdl_utils


def get_tokens_starting_with_token_and_ending_with_one_of_possible_tokens(lStartTokens, lEndTokens, lAllTokens, oTokenMap, bIncludeStartToken=False, bIncludeEndToken=True, bEarliestDetect=False):
    '''
    Returns a list of tokens objects which start with a token and ends

    This extract function is intended for situations in which the start of a sequence of tokens is known, but the end token can vary due to optional language constructs.

    Parameters:

      lStartTokens : list of token types

      lEndTokens : list of token types

      lAllTokens : list of token objects

      oTokenMap : token map object

      bIncludeStartToken : boolean
          True  = Include the starting bounding token from lStartTokens
          False = Exclude the starting bounding token from lStartTokens and leading whitesapce and comments

      bIncludeEndToken : boolean
          True  = Include the ending bounding token from lEndTokens
          False = Exclude the ending bounding token from lEndTokens and trailing whitespace and comments

      bEarliestDetect : boolean
          True  = the first token in the lEndTokens bounds the list of tokens
          False = the last token in the lEndTokens bounds the list of tokens
    '''
    lReturn = []

    lStartIndexes = gather_start_indexes(lStartTokens, oTokenMap, bIncludeStartToken)

    lEndIndexes, iMax = gather_end_indexes(lEndTokens, oTokenMap)

    lStartIndexes.append(iMax)

    iStart, iEnd, iStep = set_loop_parameters(bEarliestDetect, lEndTokens)

    for iIndex, iStartIndex in enumerate(lStartIndexes[:len(lStartIndexes) - 1]):
        iNextIndex = lStartIndexes[iIndex + 1]
        for i in range(iStart, iEnd, iStep):
            lEndIndexTemp = lEndIndexes[i]
            bBreak = False
            for iEndTemp in lEndIndexTemp:
                if iStartIndex < iEndTemp and iEndTemp < iNextIndex:
                    iLine = oTokenMap.get_line_number_of_index(iStartIndex)
                    if bIncludeEndToken:
                        iStartIndex, lTemp = vhdl_utils.remove_leading_whitespace_and_comments(iStartIndex, lAllTokens[iStartIndex:iEndTemp + 1])
                    else:
                        iStartIndex, lTemp = vhdl_utils.remove_leading_whitespace_and_comments(iStartIndex, lAllTokens[iStartIndex:iEndTemp])
                    lNewTemp = vhdl_utils.remove_trailing_whitespace_and_comments(lTemp)
                    lReturn.append(tokens.New(iStartIndex - 1, iLine, lNewTemp))
                    bBreak = True
                    break
            if bBreak:
                break
    return lReturn


def gather_start_indexes(lStartTokens, oTokenMap, bIncludeStartToken):
    lStartIndexesTemp = utils.get_indexes_of_token_list(lStartTokens, oTokenMap)
    if bIncludeStartToken:
        lStartIndexes = lStartIndexesTemp
    else:
        lStartIndexes = []
        for iIndex in lStartIndexesTemp:
            lStartIndexes.append(iIndex + 1)

    return lStartIndexes


def gather_end_indexes(lEndTokens, oTokenMap):
    lEndIndexes = []
    iMax = 0
    iMaxLength = 0
    for oToken in lEndTokens:
        lIndexes = oTokenMap.get_token_indexes(oToken)
        lEndIndexes.append(lIndexes)
        try:
            iMax = max(iMax, lIndexes[-1])
        except IndexError:
            iMax = iMax
        iMaxLength = max(iMaxLength, len(lIndexes))
    iMax +=1

    return lEndIndexes, iMax

def set_loop_parameters(bEarliestDetect, lEndTokens):
    if bEarliestDetect:
        iStart = 0
        iEnd = len(lEndTokens)
        iStep = 1
    else:
        iStart = len(lEndTokens) - 1
        iEnd = -1
        iStep = -1
    return iStart, iEnd, iStep




from vsg.deprecated_rule import Rule


class rule_412(Rule):
    '''
    This rule has been superceeded by rule `architecture_027 <architecture_rules.html#architecture-027>`_.
    '''

    def __init__(self):
        Rule.__init__(self, 'procedure', '412')
        self.message.append('Rule ' + self.unique_id + ' has been superceeded by rule architecture_027.')




from vsg.vhdlFile.classify import attribute_specification
from vsg.vhdlFile.classify import subprogram_body
from vsg.vhdlFile.classify import subprogram_declaration
from vsg.vhdlFile.classify import subprogram_instantiation_declaration
from vsg.vhdlFile.classify import use_clause


def detect(iToken, lObjects):
    '''
    protected_type_declarative_item ::=
        subprogram_declaration
      | subprogram_instantiation_declaration
      | attribute_specification
      | use_clause
    '''

    iReturn = subprogram_declaration.detect(iToken, lObjects)
    if iReturn != iToken:
        iReturn = subprogram_body.detect(iReturn, lObjects)
        return iReturn

    iReturn = subprogram_instantiation_declaration.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = attribute_specification.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = use_clause.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    return iToken




from vsg import parser


class others_keyword(parser.keyword):
    '''
    unique_id = instantiation_list : others_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class all_keyword(parser.keyword):
    '''
    unique_id = instantiation_list : all_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class instantiation_label(parser.label):
    '''
    unique_id = instantiation_list : process_label
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class comma(parser.comma):
    '''
    unique_id = instantiation_list : comma
    '''

    def __init__(self, sString=','):
        parser.comma.__init__(self)




from vsg.deprecated_rule import Rule


class rule_202(Rule):
    '''
    This rule has been moved to rule `subprogram_body_202 <subprogram_body_rules.html#subprogram-body-202>`_.
    '''

    def __init__(self):
        Rule.__init__(self, 'procedure', '202')
        self.message.append('Rule ' + self.unique_id + ' has been moved to rule subprogram_body_202.')




from vsg.rules import insert_token_right_of_token_if_it_does_not_exist_before_token

from vsg.token import package_declaration as token


class rule_007(insert_token_right_of_token_if_it_does_not_exist_before_token):
    '''
    This rule checks for the **package** keyword on the end package declaration.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end FIFO_PKG;

    **Fix**

    .. code-block:: vhdl

       end package FIFO_PKG;
    '''

    def __init__(self):
        insert_token_right_of_token_if_it_does_not_exist_before_token.__init__(self, 'package', '007', token.end_package_keyword('package'), token.end_keyword, token.semicolon)
        self.solution = '*package* keyword'




from vsg import parser
from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.if_statement.if_keyword, parser.open_parenthesis])


class rule_003(Rule):
    '''
    This rule checks for a single space between the **if** keyword and the (.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

      if(a = '1') then

      if   (a = '1') then

    **Fix**

    .. code-block:: vhdl

      if (a = '1') then

      if (a = '1') then
    '''
    def __init__(self):
        Rule.__init__(self, 'if', '003', lTokens)




from vsg.rules import multiline_structure as Rule

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.simple_waveform_assignment.target, token.simple_waveform_assignment.semicolon])


class rule_009(Rule):
    '''
    This rule checks the structure of multiline simple sequential signal assignments that contain arrays.

    |configuring_array_multiline_structure_rules_link|

    **Violation**

    .. code-block:: vhdl

       wr_data <= (0, 65535, 32768);

    **Fix**

    .. code-block:: vhdl

       wr_data <=
       (
         0,
         65535,
         32768
       );
    '''

    def __init__(self):
        Rule.__init__(self, 'sequential', '009', lTokenPairs)
        self.assignment_operator = token.simple_waveform_assignment.assignment
        self.semicolon = token.simple_waveform_assignment.semicolon




from vsg.rules import split_line_at_token

from vsg import token

lTokens = []
lTokens.append(token.library_clause.keyword)


class rule_007(split_line_at_token):
    '''
    This rule checks for code after the **is** keyword.

    **Violation**

    .. code-block:: vhdl

       context c1 is -- Comments are allowed

       context c1 is library ieee; -- This is not allowed

    **Fix**

    .. code-block:: vhdl

       context c1 is -- Comments are allowed

       context c1 is
         library ieee; -- This is not allowed
    '''

    def __init__(self):
        split_line_at_token.__init__(self, 'context', '007', lTokens)
        self.solution = 'Move library and code after library to the next line'




from vsg.rules import remove_excessive_blank_lines_above_line_starting_with_token

from vsg import token

lTokens = []
lTokens.append(token.if_statement.else_keyword)

lOverrides = []
lOverrides.append(token.case_statement.semicolon)
lOverrides.append(token.loop_statement.semicolon)


class rule_010(remove_excessive_blank_lines_above_line_starting_with_token):
    '''
    This rule checks for blank lines before the **else** keyword.

    **Violation**

    .. code-block:: vhdl

        d <= '1';


      else

    **Fix**

    .. code-block:: vhdl

        d <= '1';
      else
    '''
    def __init__(self):
        remove_excessive_blank_lines_above_line_starting_with_token.__init__(self, 'if', '010', lTokens, iAllow=0, lOverrides=lOverrides)
        self.solution = 'Remove blank line(s) before the *else* keyword.'
        self.configuration_documentation_link = None




from vsg import parser

from vsg.rules.whitespace_after_token import Rule

lTokens = []
lTokens.append(parser.comma)


class rule_007(Rule):
    '''
    This rule checks for spaces after a comma.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       proc : process (wr_en,rd_en,overflow) is

    **Fix**

    .. code-block:: vhdl

       proc : process (wr_en, rd_en, overflow) is
    '''

    def __init__(self):
        Rule.__init__(self, 'whitespace', '007', lTokens)
        self.number_of_spaces = '>=1'




from vsg.token import array_constraint as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import array_element_constraint
from vsg.vhdlFile.classify import index_constraint


def detect(iToken, lObjects):
    '''
    array_constraint ::=
        index_constraint [ array_element_constraint ]
      | ( open ) [ array_element_constraint ]

    '''
    if open_detected(iToken, lObjects):
        return classify(iToken, lObjects)
    if index_constraint.detect(iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def detect_discrete_subtype_indication(iToken, lObjects):
    if utils.is_next_token('(', iToken, lObjects):
        return index_constraint.classify(iToken, lObjects)
    return iToken

def classify(iToken, lObjects):

    iCurrent = utils.find_next_token(iToken, lObjects)
    if utils.is_next_token('open', iCurrent + 1, lObjects):
        iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required('open', token.open_keyword, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)
    else:
        iCurrent = index_constraint.classify(iCurrent, lObjects)

    iCurrent = array_element_constraint.detect(iCurrent, lObjects)

    return iCurrent


def open_detected(iToken, lObjects):
    if utils.is_next_token('(', iToken, lObjects):
        if utils.find_in_next_n_tokens('open', 2, iToken, lObjects):
            return True
    return False


def classify_index_constraint(iToken, lObjects):
    print('--> classify_index_constraint')
    return index_constraint.classify(iToken, lObjects)




from vsg import parser
from vsg import violation

from vsg.vhdlFile import utils
from vsg.rule_group import structure
from vsg.rules import utils as rules_utils

from vsg.rules.insert_token_next_to_token_if_it_does_not_exist_between_tokens_using_value_from_token import insert_token_next_to_token_if_it_does_not_exist_between_tokens_using_value_from_token as Rule


class insert_token_right_of_token_if_it_does_not_exist_between_tokens_using_value_from_token(Rule):
    '''
    Checks for the existence of a token and will insert it if it does not exist.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    insert_token : token object
       token to insert if it does not exist.

    anchor_token : token object
       token to check if insert_token exists to the right of

    value_token : token object
       token to pull the value from
    '''

    def __init__(self, name, identifier, insert_token, anchor_token, left_token, right_token, value_token):
        Rule.__init__(self, name=name, identifier=identifier)
        self.insert_token = insert_token
        self.anchor_token = anchor_token
        self.left_token = left_token
        self.right_token = right_token
        self.value_token = value_token
        self.configuration_documentation_link = 'configuring_optional_items_link'
        self.direction = 'right'




from vsg import deprecated_rule


class rule_032(deprecated_rule.Rule):
    '''
    This rule has been replaced with the following rules:

    * `process_037 <process_rules.html#process-037>`_
    * `process_038 <process_rules.html#process-038>`_
    '''

    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'process', '032')
        self.message.append('Rule ' + self.unique_id + ' has been replaced with the following rules:')
        self.message.append('  * process_037')
        self.message.append('  * process_038')




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.ieee.std_logic_1164.types.std_logic)
lTokens.append(token.ieee.std_logic_1164.types.std_logic_vector)
lTokens.append(token.ieee.std_logic_1164.types.std_ulogic)
lTokens.append(token.ieee.std_logic_1164.types.std_ulogic_vector)
lTokens.append(token.ieee.std_logic_1164.types.integer)
lTokens.append(token.ieee.std_logic_1164.types.signed)
lTokens.append(token.ieee.std_logic_1164.types.unsigned)


class rule_500(token_case):
    '''
    This rule checks IEEE types have the proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       port (
         WR_EN    : in    STD_LOGIC;
         RD_EN    : in    STD_logic;
         DATA     : inout STD_LOGIC_VECTOR(31 downto 0)
       );

    **Fix**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         DATA     : inout std_logic_vector(31 downto 0)
       );
    '''

    def __init__(self):
        token_case.__init__(self, 'ieee', '500', lTokens)
        self.groups.append('case::keyword')




from .rule_001 import rule_001
from .rule_002 import rule_002
from .rule_003 import rule_003
from .rule_004 import rule_004
from .rule_005 import rule_005

from .rule_007 import rule_007
from .rule_008 import rule_008
from .rule_009 import rule_009

from .rule_200 import rule_200




from vsg import parser


class register_keyword(parser.keyword):
    '''
    unique_id = signal_kind : register_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class bus_keyword(parser.keyword):
    '''
    unique_id = signal_kind : bus_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg.vhdlFile import utils


def detect(iToken, lObjects):
    '''
    range ::=
        range_attribute_name
      | simple_expression direction simple_expression
    '''
    if check_for_range_attribute_name(iToken, lObjects):
        return True
    return detect_direction(iToken, lObjects)


def check_for_range_attribute_name(iToken, lObjects):
    iParens = 0
    for iIndex in range(iToken, len(lObjects)):
        iParens = utils.update_paren_counter(iIndex, lObjects, iParens)

        if token_is_matching_close_parenthesis(iParens):
            return False
        if token_is_tic(iParens, iIndex, lObjects):
            return True

    return False


def token_is_matching_close_parenthesis(iParens):
    if iParens == -1:
        return True
    return False


def check_for_todo_token(iIndex, lObjects):
    if utils.token_is_whitespace_or_comment(lObjects[iIndex]):
        return False
    return True


def token_is_tic(iParens, iIndex, lObjects):
    if iParens == 0 and utils.object_value_is(lObjects, iIndex, "'"):
        return True


def detect_direction(iToken, lObjects):
    iParens = 0
    for iIndex in range(iToken, len(lObjects)):
        iParens = utils.update_paren_counter(iIndex, lObjects, iParens)
        if iParens == -1:
            return False
        if check_for_direction(iParens, iIndex, lObjects):
            return True
    return False


def check_for_direction(iParens, iIndex, lObjects):
    if iParens == 0 and utils.is_next_token_one_of(['downto', 'to'], iIndex, lObjects):
        return True
    return False




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.component_declaration.end_keyword)


class rule_009(token_indent):
    '''
    This rule checks the indent of the **end component** keywords.

    **Violation**

    .. code-block:: vhdl

          overflow : std_logic
        );
            end component fifo;

    **Fix**

    .. code-block:: vhdl

           overflow : std_logic
         );
       end component fifo;
    '''

    def __init__(self):
        token_indent.__init__(self, 'component', '009', lTokens)




lSingleCharacterSymbols = [',', ':', '(', ')', '\'', '"', '+', '&', '-', '*', '/', '<', '>', ';', '=', '[', ']', '?']
lTwoCharacterSymbols = ['=>','**', ':=', '/=', '>=', '<=', '<>', '??', '?=', '?<', '?>', '<<', '>>', '--', '/*', '*/']
lThreeCharacterSymbols = ['?/=', '?<', '?<=', '?>=']
lFourCharacterSymbols = ['\\?=\\']

lStopChars = [' ', '(', ';']


def create(sString):
    '''
    This function takes a string and returns a list of tokens.
    '''

    oLine = New(sString)
    oLine.combine_whitespace()
    oLine.combine_string_literals()
    oLine.combine_backslash_characters_into_symbols()
    oLine.combine_two_character_symbols()
    oLine.combine_characters_into_words()
    oLine.combine_character_literals()
    oLine.combine_comments()
    oLine.split_natural_numbers()
    return oLine.lChars


class New():
    def __init__(self, sLine):
        self.lChars = convert_string_to_chars(sLine)

    def combine_whitespace(self):
        lReturn = []
        sSpace = ''
        for sChar in self.lChars:
            if sChar.isspace():
                sSpace += sChar
            else:
                if sSpace.isspace():
                    lReturn.append(sSpace)
                    sSpace = ''
                lReturn.append(sChar)

        lReturn.append(sSpace)

        self.lChars = lReturn

    def combine_backslash_characters_into_symbols(self):
        lReturn = []
        sSymbol = ''
        bSymbol = False
        for sChar in self.lChars:
            if stop_character_found(sChar, bSymbol):
                bSymbol = False
                lReturn.append(sSymbol)
                sSymbol = ''
            bSymbol = inside_backslash_symbol(bSymbol, sChar)
            sSymbol = append_to_symbol(bSymbol, sSymbol, sChar)
            lReturn = append_to_list(bSymbol, lReturn, sChar)
        lReturn = add_trailing_string(lReturn, sSymbol)
        self.lChars = lReturn

    def combine_two_character_symbols(self):
        lReturn = []
        sNextChar = ''
        bSkip = False
        for iChar, sChar in enumerate(self.lChars):
            if bSkip:
                bSkip = False
                continue
            try:
                sNextChar = self.lChars[iChar + 1]
            except IndexError:
                sNextChar = ''
            if sChar + sNextChar in lTwoCharacterSymbols:
                bSkip = True
                lReturn.append(sChar + sNextChar)
            else:
                lReturn.append(sChar)
        self.lChars = lReturn

    def combine_characters_into_words(self):
        lReturn = []
        sTemp = ''
        for sChar in self.lChars:
            if character_is_part_of_word(sChar):
                sTemp += sChar
            else:
                if sTemp != '':
                    lReturn.append(sTemp)
                lReturn.append(sChar)
                sTemp = ''

        if len(sTemp) != 0:
            lReturn.append(sTemp)

        self.lChars = lReturn

    def combine_string_literals(self):
        lQuotePairs = find_indexes_of_double_quote_pairs(self.lChars)
        lQuotePairs.reverse()

        combine_quote_pairs(lQuotePairs, self)

    def combine_character_literals(self):
        lQuotes = find_indexes_of_token_with_value("'", self.lChars)
        lLiterals = find_character_literal_candidates(lQuotes, self.lChars)
        if len(lLiterals) == 0:
            return None
        lQuotePairs = filter_character_literal_candidates(lLiterals)
        lQuotePairs.reverse()

        combine_quote_pairs(lQuotePairs, self)

    def combine_comments(self):
        if has_trailing_whitespace(self.lChars):
            combine_comment_with_trailing_whitespace(self)
        else:
            combine_comment(self)

    def split_natural_numbers(self):
        lReturn = []
        for sChar in self.lChars:
            if is_natural_number(sChar):
                lReturn.extend(parse_natural_number(sChar))
            else:
                lReturn.append(sChar)

        self.lChars = lReturn


def is_natural_number(sString):
    lString = sString.lower().split('e')
    lBase = lString[0].split('.')
    lBase.extend(lString[1:])
    for sNum in lBase[0:-1]:
        if not sNum.isdigit():
            return False
    return True


def parse_natural_number(sString):
    lReturn = []
    sTemp = ''
    for sChar in sString:
        if sChar.lower() == 'e':
            lReturn.append(sTemp)
            lReturn.append(sChar)
            sTemp = ''
        else:
            sTemp += sChar
    if len(sTemp) > 0:
        lReturn.append(sTemp)
    return lReturn


def combine_quote_pairs(lQuotePairs, self):
    for lPair in lQuotePairs:
        iLeft = lPair[0]
        iRight = lPair[1] + 1
        lReturn = self.lChars[0:iLeft]
        lReturn.append(''.join(self.lChars[iLeft:iRight]))
        lReturn.extend(self.lChars[iRight:])
        self.lChars = lReturn


def find_character_literal_candidates(lQuotes, lChars):
    lReturn = []
    for iIndex, iQuote in enumerate(lQuotes[0:-1]):
        if is_character_literal_candidate(iIndex, lQuotes, lChars):
            lReturn.append([iQuote, iQuote + 2])
    return lReturn


def is_character_literal_candidate(iIndex, lQuotes, lChars):
    iQuote = lQuotes[iIndex]
    if there_is_a_single_token_between_quotes(iIndex, lQuotes) and \
       token_between_quotes_is_a_single_character(iQuote, lChars):
        return True
    return False


def there_is_a_single_token_between_quotes(iIndex, lQuotes):
    if lQuotes[iIndex] + 2 == lQuotes[iIndex + 1]:
        return True
    return False


def token_between_quotes_is_a_single_character(iQuote, lChars):
    if len(lChars[iQuote + 1]) == 1:
        return True
    return False


def filter_character_literal_candidates(lLiterals):
    lReturn = []
    for iIndex, lLiteral in enumerate(lLiterals[0:-1]):
        lNextLiteral = lLiterals[iIndex + 1]
        if lLiteral[1] != lNextLiteral[0]:
            lReturn.append(lLiteral)
    lReturn.append(lLiterals[-1])
    return lReturn


def combine_comment_with_trailing_whitespace(self):
    if self.lChars.count('--') > 0:
        iIndex = self.lChars.index('--')
        lReturn = self.lChars[0:iIndex]
        lReturn.append(''.join(self.lChars[iIndex:-1]))
        lReturn.append(self.lChars[-1])
        self.lChars = lReturn


def combine_comment(self):
    if self.lChars.count('--') > 0:
        iIndex = self.lChars.index('--')
        lReturn = self.lChars[0:iIndex]
        lReturn.append(''.join(self.lChars[iIndex::]))
        self.lChars = lReturn


def has_trailing_whitespace(lChars):
    if len(lChars) == 0:
        return False
    if lChars[-1].isspace():
        return True
    return False


def inside_backslash_symbol(bSymbol, sChar):
    if backslash_character_found(sChar):
        return True
    return bSymbol


def append_to_symbol(bSymbol, sSymbol, sChar):
    if bSymbol:
        return sSymbol + sChar
    return sSymbol


def append_to_list(bSymbol, lChars, sChar):
    lReturn = lChars
    if not bSymbol:
        lReturn.append(sChar)
    return lReturn


def backslash_character_found(sChar):
    if sChar == '\\':
        return True
    return False


def stop_character_found(sChar, bLiteral):
    if (sChar in lStopChars or ' ' in sChar) and bLiteral:
        return True
    return False


def add_trailing_string(lReturn, sString):
    if len(sString) > 0:
        lReturn.append(sString)
    return lReturn


def convert_string_to_chars(sString):
    lReturn = []
    for sChar in sString:
        lReturn.append(sChar)
    return lReturn


def character_is_part_of_word(sChar):
    if len(sChar) > 1:
        return False
    elif sChar.isspace():
        return False
    elif sChar in lSingleCharacterSymbols:
        return False
    return True


def find_indexes_of_double_quote_pairs(lTokens):
    lReturn = []
    lIndexes = find_indexes_of_token_with_value('"', lTokens)
    for iIndex in range(0, len(lIndexes) - 1, 2):
        lReturn.append([lIndexes[iIndex], lIndexes[iIndex + 1]])
    return lReturn


def find_indexes_of_token_with_value(sValue, lTokens):
    lReturn = []
    for iToken, sToken in enumerate(lTokens):
        if sToken == sValue:
            lReturn.append(iToken)
    return lReturn




from vsg import parser


class type_keyword(parser.keyword):
    '''
    unique_id = incomplete_type_declaration : type_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class identifier(parser.identifier):
    '''
    unique_id = incomplete_type_declaration : identifier
    '''

    def __init__(self, sString):
        parser.identifier.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = incomplete_type_declaration : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg import parser


class comma(parser.comma):
    '''
    unique_id = group_constituent_list : comma
    '''

    def __init__(self, sString=','):
        parser.comma.__init__(self)


class group_constituent(parser.item):
    '''
    unique_id = group_constituent_list : group_constituent
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)





from vsg.rules import token_indent_between_tokens

from vsg import token

lTokens = []
lTokens.append(token.interface_constant_declaration.constant_keyword)
lTokens.append(token.interface_signal_declaration.signal_keyword)
lTokens.append(token.interface_variable_declaration.variable_keyword)
lTokens.append(token.interface_file_declaration.file_keyword)
lTokens.append(token.interface_unknown_declaration.identifier)
lTokens.append(token.interface_incomplete_type_declaration.type_keyword)
lTokens.append(token.interface_procedure_specification.procedure_keyword)
lTokens.append(token.interface_function_specification.function_keyword)
lTokens.append(token.interface_package_declaration.package_keyword)

oStart = token.function_specification.open_parenthesis
oEnd = token.function_specification.close_parenthesis


class rule_008(token_indent_between_tokens):
    '''
    This rule checks the indent of function parameters on multiple lines.

    **Violation**

    .. code-block:: vhdl

       function func_1 (a : integer; b : integer;
                   c : unsigned(3 downto 0);
           d : std_logic_vector(7 downto 0);
              e : std_logic) return integer is
       begin

       end;

    **Fix**

    .. code-block:: vhdl

       function func_1 (a : integer; b : integer;
         c : unsigned(3 downto 0);
         d : std_logic_vector(7 downto 0);
         e : std_logic) return integer is
       begin

       end;
    '''

    def __init__(self):
        token_indent_between_tokens.__init__(self, 'function', '008', lTokens, oStart, oEnd)




from vsg import deprecated_rule


class rule_001(deprecated_rule.Rule):
    '''
    This rule has been superceeded by:

    * `attribute_declaration_300 <attribute_declaration_rules.html#attribute-declaration-300>`_
    * `attribute_specification_300 <attribute_specification_rules.html#attribute-specification-300>`_
    '''

    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'attribute', '001')
        self.message.append('Rule ' + self.unique_id + ' has been superceeded by attribute_declaration_300 and attribute_specification_300.')





class New():

  def __init__(self, iLine, oTokens, sSolution=None):
      self.iLine = iLine
      self.oTokens = oTokens
      self.sSolution = sSolution
      self.action = None
      self.remap = False
      self.fix_blank_lines = False

  def get_tokens(self):
      return self.oTokens.get_tokens()

  def get_line_number(self):
      return self.iLine

  def set_tokens(self, lTokens):
      self.oTokens.set_tokens(lTokens)

  def set_solution(self, sSolution):
      self.solution = sSolution

  def get_solution(self):
      return self.sSolution

  def set_action(self, sAction):
      self.action = sAction

  def get_action(self):
      return self.action

  def get_token_value(self):
      return self.oTokens.get_token_value()

  def has_code_tag(self, sCodeTag):
      try:
          for oToken in self.oTokens.get_tokens():
              if oToken.has_code_tag(sCodeTag):
                  return True
          return False
#          return self.oTokens.get_tokens()[0].has_code_tag(sCodeTag)
      except IndexError:
          return False
      except AttributeError:
          return False

  def set_remap(self):
      self.remap = True

  def get_remap(self):
      return self.remap

  def fix_blanks(self):
      return self.fix_blank_lines

  def get_start_index(self):
      return self.oTokens.get_start_index()




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.case_generate_statement.end_keyword)
lTokens.append(token.for_generate_statement.end_keyword)
lTokens.append(token.if_generate_statement.end_keyword)


class rule_007(token_indent):
    '''
    This rule checks the indent of the **end generate** keyword.

    **Violation**

    .. code-block:: vhdl

       ram_array : for i in 0 to 7 generate
       begin
         end generate ram_array;

    **Fix**

    .. code-block:: vhdl

       ram_array : for i in 0 to 7 generate
       begin
       end generate ram_array;
    '''

    def __init__(self):
        token_indent.__init__(self, 'generate', '007', lTokens)




from vsg.rules import align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens

from vsg import parser
from vsg import token

lAlign = []
lAlign.append(parser.comment)

oStart = token.process_statement.begin_keyword
oEnd = token.process_statement.end_keyword

lSkip = []
lSkip.append(parser.comment)


class rule_035(align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens):
    '''
    This rule checks the alignment of inline comments between the process begin and end process lines.
    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       proc_1: process () is
       begin

         a <= '1';   -- Assert
         b <= '0';       -- Deassert
         c <= '1'; -- Enable

       end process proc_1;

    **Fix**

    .. code-block:: vhdl

       proc_1: process () is
       begin

         a <= '1'; -- Assert
         b <= '0'; -- Deassert
         c <= '1'; -- Enable

       end process proc_1;
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens.__init__(self, 'process', '035', lAlign, oStart, oEnd, lSkip)
        self.solution = 'Align comment.'
        self.subphase = 3
        self.blank_line_ends_group = False
        self.comment_line_ends_group = False
        self.compact_alignment = False
        self.include_lines_without_comments = True




from vsg.rules import remove_tokens_bounded_by_tokens_and_remove_trailing_whitespace

from vsg.token import concurrent_signal_assignment_statement as token


class rule_005(remove_tokens_bounded_by_tokens_and_remove_trailing_whitespace):
    '''
    This rule checks for labels on concurrent assignments.
    Labels on concurrents are optional and do not provide additional information.

    **Violation**

    .. code-block:: vhdl

       WR_EN_OUTPUT : WR_EN <= q_wr_en;
       RD_EN_OUTPUT : RD_EN <= q_rd_en;

    **Fix**

    .. code-block:: vhdl

       WR_EN <= q_wr_en;
       RD_EN <= q_rd_en;
    '''

    def __init__(self):
        remove_tokens_bounded_by_tokens_and_remove_trailing_whitespace.__init__(self, 'concurrent', '005', token.label_name, token.label_colon)
        self.solution = 'Remove Label'




from vsg.token import port_map_aspect as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import association_list


def detect(iToken, lObjects):
    '''
    port_map_aspect ::=
        port map ( *port*_association_list )
    '''

    if utils.are_next_consecutive_tokens(['port', 'map', '('], iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('port', token.port_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('map', token.map_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iCurrent, lObjects)

    iCurrent = association_list.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)

    return iCurrent




from vsg import parser
from vsg import violation

from vsg.rules import utils
from vsg.rule_group import whitespace


class rule_100(whitespace.Rule):
    '''
    This rule checks for a single space after the **--**.

    |configuring_whitespace_after_comment_rules_link|

    **Violation**

    .. code-block:: vhdl

       --Comment 1
       --|Comment 2
       ---Comment
       ---------------------------

    **Fix**

    .. code-block:: vhdl

       -- Comment 1
       --|Comment 2
       ---Comment
       ---------------------------
    '''

    def __init__(self):
        whitespace.Rule.__init__(self, name='comment', identifier='100')
        self.solution = 'Undefined'
        self.phase = 2
        self.disable = False
        self.lTokens = [parser.comment]
        self.exceptions = ['--!', '--=', '--+', '--|', '---']
        self.configuration.append('exceptions')
        self.patterns = ['--!', '--|']
        self.configuration.append('patterns')
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        lReturn = []
        lToi = oFile.get_tokens_matching(self.lTokens)
        for oToi in lToi:
            oToken = oToi.get_tokens()[0]
            if oToken.is_block_comment:
                continue
            if self.pattern_found(oToken):
                oToi.set_meta_data('type', 'pattern')
                oToi.set_meta_data('pattern', self.get_matching_pattern(oToken))
                lReturn.append(oToi)
            elif not self.valid_comment(oToken):
                oToi.set_meta_data('type', 'exception')
                lReturn.append(oToi)
        return lReturn

    def _analyze(self, lToi):
        for oToi in lToi:
            create_violation(self, oToi)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()

        sToken = lTokens[0].get_value()
        sNewToken = sToken[0:dAction['index']] + ' ' + sToken[dAction['index']:]
        lTokens[0].set_value(sNewToken)
        oViolation.set_tokens(lTokens)

    def valid_comment(self, oToken):
        sToken = oToken.get_value()
        if len(sToken) == 2:
            return True
        if sToken.startswith('-- '):
            return True
        if self.exception_found(sToken):
            return True
        return False

    def exception_found(self, sToken):
        if sToken[0:3] in self.exceptions:
            return True
        return False

    def pattern_found(self, oToken):
        sToken = oToken.get_value()
        if len(sToken) < 4:
            return False
        for sPattern in self.patterns:
            if sToken.startswith(sPattern):
                if not sToken.startswith(sPattern + ' '):
                    return True
        return False

    def get_matching_pattern(self, oToken):
        sToken = oToken.get_value()
        for sPattern in self.patterns:
            if sToken.startswith(sPattern):
                return sPattern
        return None


def create_violation_action_dict(sToken, iIndex):
    dReturn = {}
    dReturn['violation'] = True
    dReturn['index'] = iIndex
    dReturn['solution'] = create_solution(iIndex, sToken)
    return dReturn


def create_violation(self, oToi):
    if oToi.get_meta_data('type') == 'pattern':
        iIndex = len(oToi.get_meta_data('pattern'))
    else:
        iIndex = 2
    dResults = create_violation_action_dict(oToi.get_tokens()[0].get_value(), iIndex)
    oViolation = violation.New(oToi.get_line_number(), oToi, dResults['solution'])
    oViolation.set_action(dResults)
    self.add_violation(oViolation)


def create_solution(iIndex, sComment):
    return 'Change "' + sComment[0:iIndex + 1] + '" to "' + sComment[0:iIndex] + ' ' + sComment[iIndex] + '"'




from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import structure
from vsg.vhdlFile import utils


class remove_carriage_return_after_token(structure.Rule):
    '''
    Checks for a carriage return within the next two tokens after the one given.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of parser object types
       object types to check the prefix

    lPrefixes : string list
       acceptable prefixes
    '''

    def __init__(self, name, identifier, lTokens, bInsertSpace=False):
        structure.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.bInsertSpace = bInsertSpace
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_n_tokens_after_token(4, self.lTokens)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            for iToken, oToken in enumerate(lTokens[:len(lTokens)]):
                if iToken < 3:
                    if isinstance(oToken, parser.carriage_return):
                        oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
                        self.add_violation(oViolation)
                        break

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()

        lTokens = utils.remove_carriage_returns_from_token_list(lTokens)
        lTokens = utils.remove_consecutive_whitespace_tokens(lTokens)
        if self.bInsertSpace:
            if not isinstance(lTokens[1], parser.whitespace):
                rules_utils.insert_whitespace(lTokens, 1)

        oViolation.set_tokens(lTokens)




from vsg.vhdlFile.classify import access_type_definition
from vsg.vhdlFile.classify import composite_type_definition
from vsg.vhdlFile.classify import file_type_definition
from vsg.vhdlFile.classify import protected_type_definition
from vsg.vhdlFile.classify import scalar_type_definition


def detect(iToken, lObjects):
    '''
    type_definition ::=
        scalar_type_definition
      | composite_type_definition
      | access_type_definition
      | file_type_definition
      | protected_type_definition
    '''

    iReturn = scalar_type_definition.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = access_type_definition.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = composite_type_definition.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = file_type_definition.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = protected_type_definition.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    return iToken




from vsg import parser
from vsg import violation

from vsg.rule_group.blank_line import Rule
from vsg.vhdlFile import utils


class rule_200(Rule):
    '''
    This rule enforces a maximum number of consecutive blank lines.

    |configuring_consecutive_blank_line_rules|

    **Violation**

    .. code-block:: vhdl

       a <= b;


       c <= d;

    **Fix**

    .. code-block:: vhdl

       a <= b;

       c <= d;
    '''

    def __init__(self):
        Rule.__init__(self, name='whitespace', identifier='200')
        self.blank_lines_allowed = 1
        self.configuration.append('blank_lines_allowed')
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return [oFile.get_all_tokens()]

    def _analyze(self, lToi):
        oToi = lToi[0]
        iLine, lTokens = utils.get_toi_parameters(oToi)
        iCount = 0
        for iToken, oToken in enumerate(lTokens[:len(lTokens) - 1]):
            iLine = utils.increment_line_number(iLine, oToken)

            if isinstance(oToken, parser.blank_line):
                if iCount == 0:
                    iLineNumber = iLine
                    iStartToken = iToken
                iCount += 1

            if isinstance(oToken, parser.carriage_return):
                if not isinstance(lTokens[iToken + 1], parser.blank_line):
                    if iCount > self.blank_lines_allowed:
                        sSolution = 'Remove ' + str(iCount - self.blank_lines_allowed) + ' blank line(s) below'
                        lExtractedTokens = oToi.extract_tokens(iStartToken, iToken)
                        oViolation = violation.New(iLineNumber, lExtractedTokens, sSolution)
                        dAction = {}
                        dAction['remove'] = iCount - self.blank_lines_allowed
                        oViolation.set_action(dAction)
                        self.add_violation(oViolation)

                    iCount = 0

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        lTokens = lTokens[2*dAction['remove']:]
        oViolation.set_tokens(lTokens)




from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import context_item


def detect(iToken, lObjects):
    '''
    context_clause ::=
        { context_item }
    '''
    iCurrent = utils.detect_submodule(iToken, lObjects, context_item)
    return iCurrent




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.incomplete_type_declaration.type_keyword)
lTokens.append(token.full_type_declaration.type_keyword)


class rule_002(token_case):
    '''
    This rule checks the **type** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       TYPE state_machine is (idle, write, read, done);

    **Fix**

    .. code-block:: vhdl

       type state_machine is (idle, write, read, done);
    '''

    def __init__(self):
        token_case.__init__(self, 'type', '002', lTokens)
        self.groups.append('case::keyword')




from vsg.rules import token_suffix

from vsg import token

lTokens = []
lTokens.append(token.variable_declaration.identifier)


class rule_600(token_suffix):
    '''
    This rule checks for valid suffix on variable identifiers.
    The default variable suffix is *\_v*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       variable my_var : natural;

    **Fix**

    .. code-block:: vhdl

       variable my_var_v : natural;
    '''

    def __init__(self):
        token_suffix.__init__(self, 'variable', '600', lTokens)
        self.suffixes = ['_v']
        self.solution = 'Variable identifiers'




from vsg import parser

from vsg.rules.whitespace_between_tokens import Rule

from vsg.token import case_statement as token


class rule_002(Rule):
    '''
    This rule checks for a single space after the **case** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       case    data is


    **Fix**

    .. code-block:: vhdl

       case data is
    '''
    def __init__(self):
        Rule.__init__(self, 'case', '002')
        self.left_token = token.case_keyword
        self.right_token = parser.todo




from vsg.rules import insert_token_right_of_possible_tokens_if_it_does_not_exist_before_token

from vsg.token import block_statement as token

oInsertToken = token.is_keyword('is')

lRightTokens = []
lRightTokens.append(token.block_keyword)
lRightTokens.append(token.guard_close_parenthesis)

oBeforeToken = token.begin_keyword


class rule_002(insert_token_right_of_possible_tokens_if_it_does_not_exist_before_token):
    '''
    This rule checks for the existence of the **is** keyword.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       block_label : block
       block_label : block (guard_condition)

    **Fix**

    .. code-block:: vhdl

       block_label : block is
       block_label : block (guard_condition) is
    '''

    def __init__(self):
        insert_token_right_of_possible_tokens_if_it_does_not_exist_before_token.__init__(self, 'block', '002', oInsertToken, lRightTokens, oBeforeToken)
        self.solution = '*is* keyword'
        self.groups.append('structure::optional')




from vsg.token import signal_assignment_statement as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import conditional_signal_assignment
from vsg.vhdlFile.classify import selected_signal_assignment
from vsg.vhdlFile.classify import simple_signal_assignment


def detect(iToken, lObjects):
    '''
    signal_assignment_statement ::=
        [ label : ] simple_signal_assignment
      | [ label : ] conditional_signal_assignment
      | [ label : ] selected_signal_assignment
    '''

    iCurrent = iToken

    if selected_signal_assignment.detect(iToken, lObjects):
        iCurrent = utils.tokenize_label(iCurrent, lObjects, token.label, token.label_colon)
        iCurrent = selected_signal_assignment.classify(iCurrent, lObjects)

    elif conditional_signal_assignment.detect(iToken, lObjects):
        iCurrent = utils.tokenize_label(iCurrent, lObjects, token.label, token.label_colon)
        iCurrent = conditional_signal_assignment.classify(iCurrent, lObjects)

    elif simple_signal_assignment.detect(iToken, lObjects):
        iCurrent = utils.tokenize_label(iCurrent, lObjects, token.label, token.label_colon)
        iCurrent = simple_signal_assignment.classify(iCurrent, lObjects)

    return iCurrent




from vsg import token

from vsg.rules.whitespace_between_token_pairs_bounded_by_tokens import Rule

lTokens = []
lTokens.append([token.association_element.assignment, token.association_element.actual_part])

lStart = token.generic_map_aspect.open_parenthesis
lEnd = token.generic_map_aspect.close_parenthesis


class rule_007(Rule):
    '''
    This rule checks for a single space after the **=>** keyword in generic maps.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       generic map
       (
         WIDTH =>    32,
         DEPTH => 512
       )

    **Fix**

    .. code-block:: vhdl

       generic map
       (
         WIDTH => 32,
         DEPTH => 512
       )
    '''
    def __init__(self):
        Rule.__init__(self, 'generic_map', '007', lTokens, lStart, lEnd)




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.generic_clause.generic_keyword, token.generic_clause.open_parenthesis])


class rule_003(Rule):
    '''
    This rule checks for a single space between the **generic** keyword and the (.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       generic    (

       generic(

    **Fix**

    .. code-block:: vhdl

       generic (

       generic (
    '''
    def __init__(self):
        Rule.__init__(self, 'generic', '003', lTokens)




from vsg.rules import token_case as Rule

from vsg import token

lTokens = []
lTokens.append(token.selected_waveforms.when_keyword)
lTokens.append(token.selected_expressions.when_keyword)


class rule_503(Rule):
    '''
    This rule checks the **when** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       with mux_sel select addr <=
         "0000" WHEN 0,
         "0001" WHEN 1,
         "1111" WHEN others;

    **Fix**

    .. code-block:: vhdl

       with mux_sel select addr <=
         "0000" when 0,
         "0001" when 1,
         "1111" when others;
    '''

    def __init__(self):
        Rule.__init__(self, 'selected_assignment', '503', lTokens)
        self.groups.append('case::keyword')




from vsg.rules import token_case_with_prefix_suffix

from vsg.token import architecture_body as token

lTokens = []
lTokens.append(token.entity_name)


class rule_014(token_case_with_prefix_suffix):
    '''
    This rule checks the case of the entity name in the architecture declaration.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       architecture rtl of FIFO is

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'architecture', '014', lTokens)
        self.groups.append('case::name')




from vsg import token

from vsg.rules import remove_carriage_return_after_token as Rule

lTokens = []
lTokens.append(token.concurrent_selected_signal_assignment.with_keyword)
lTokens.append(token.selected_force_assignment.with_keyword)
lTokens.append(token.selected_variable_assignment.with_keyword)
lTokens.append(token.selected_waveform_assignment.with_keyword)


class rule_001(Rule):
    '''
    This rule checks the **with** keyword is on the same line as the expression.

    **Violation**

    .. code-block:: vhdl

       with
         mux_sel select addr <=
         "0000" when 0,
         "0001" when 1,
         "1111" when others;

    **Fix**

    .. code-block:: vhdl

       with mux_sel select addr <=
         "0000" when 0,
         "0001" when 1,
         "1111" when others;
    '''

    def __init__(self):
        Rule.__init__(self, 'selected_assignment', '001', lTokens, bInsertSpace=True)
        self.solution = 'Removed carraige returns after with keyword'




from vsg.rules.whitespace_after_token import Rule

from vsg import token

lTokens = []
lTokens.append(token.constant_declaration.colon)


class rule_005(Rule):
    '''
    This rule checks for a single space after the colon.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       constant size  :integer := 1;
       constant width :     integer := 32;

    **Fix**

    .. code-block:: vhdl

       constant size  : integer := 1;
       constant width : integer := 32;
    '''

    def __init__(self):
        Rule.__init__(self, 'constant', '005', lTokens)




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.generic_map_aspect.map_keyword, token.generic_map_aspect.open_parenthesis])
lTokens.append([token.port_map_aspect.map_keyword, token.port_map_aspect.open_parenthesis])


class rule_006(Rule):
    '''
    This rule checks for a single space between the **map** keyword and the (.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       generic map(

       generic map   (

    **Fix**

    .. code-block:: vhdl

       generic map (

       generic map (
    '''
    def __init__(self):
        Rule.__init__(self, 'generic_map', '006', lTokens)
        self.solution = 'Ensure a single space exists between "map" and (.'




#from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import assertion_statement
from vsg.vhdlFile.classify import case_statement
from vsg.vhdlFile.classify import exit_statement
from vsg.vhdlFile.classify import if_statement
from vsg.vhdlFile.classify import loop_statement
from vsg.vhdlFile.classify import next_statement
from vsg.vhdlFile.classify import null_statement
from vsg.vhdlFile.classify import procedure_call_statement
from vsg.vhdlFile.classify import report_statement
from vsg.vhdlFile.classify import return_statement
from vsg.vhdlFile.classify import signal_assignment_statement
from vsg.vhdlFile.classify import variable_assignment_statement
from vsg.vhdlFile.classify import wait_statement


def detect(iToken, lObjects):
    '''
    sequential_statement ::=
        wait_statement
      | assertion_statement
      | report_statement
      | signal_assignment_statement
      | variable_assignment_statement
      | procedure_call_statement
      | if_statement
      | case_statement
      | loop_statement
      | next_statement
      | exit_statement
      | return_statement
      | null_statement
    '''

#    print('--> checking wait_statement')
    iReturn = wait_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking assertion_statement')
    iReturn = assertion_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking report_statement')
    iReturn = report_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking case_statement')
    iReturn = case_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking if_statement')
    iReturn = if_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking loop_statement')
    iReturn = loop_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking variable_assignment')
    iReturn = variable_assignment_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking exit_statement')
    iReturn = exit_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking signal_assignment')
    iReturn = signal_assignment_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking procedure_call_statement')
    iReturn = procedure_call_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking next_statement')
    iReturn = next_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking return_statement')
    iReturn = return_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

#    print('--> checking null_statement')
    iReturn = null_statement.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn
#    print('<-- Returning')
#    utils.print_next_token(iToken, lObjects)
    return iToken




from vsg import parser


class sign(parser.keyword):
    '''
    unique_id = sign : sign
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class plus(sign):
    '''
    unique_id = sign : plus
    '''

    def __init__(self, sString='+'):
        sign.__init__(self, '+')


class minus(sign):
    '''
    unique_id = sign : minus
    '''

    def __init__(self, sString='-'):
        sign.__init__(self, '-')




from vsg import token

from vsg.rules.whitespace_after_tokens_in_between_tokens import Rule

lTokens = []
lTokens.append(token.association_element.assignment)

oStart = token.procedure_call.open_parenthesis
oEnd = token.procedure_call.close_parenthesis


class rule_101(Rule):
    '''
    This rule checks for a single space after the **=>** operator in procedure calls.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       connect_ports(
         port_1 =>    data,
         port_2 =>enable,
         port_3 =>  overflow,
         port_4 => underflow
       );

    **Fix**

    .. code-block:: vhdl

       connect_ports(
         port_1 => data,
         port_2 => enable,
         port_3 => overflow,
         port_4 => underflow
       );
    '''
    def __init__(self):
        Rule.__init__(self, 'procedure_call', '101', lTokens, oStart, oEnd)




from vsg.rules import multiline_alignment_between_tokens

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.process_statement.open_parenthesis, token.process_statement.close_parenthesis])


class rule_020(multiline_alignment_between_tokens):
    '''
    This rule checks the indentation of multiline sensitivity lists.

    |configuring_multiline_indent_rules_link|

    **Violation**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                            rd_full, wr_full,
                   overflow, underflow
                        ) is begin

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full,
                         overflow, underflow
                        ) is
       begin
    '''

    def __init__(self):
        multiline_alignment_between_tokens.__init__(self, 'process', '020', lTokenPairs, True)




from vsg import parser


class alternative_label_name(parser.label):
    '''
    unique_id = case_generate_alternative : alternative_label_name
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class alternative_label_colon(parser.label_colon):
    '''
    unique_id = case_generate_alternative : alternative_label_colon
    '''

    def __init__(self):
        parser.label_colon.__init__(self)


class when_keyword(parser.keyword):
    '''
    unique_id = case_generate_alternative : when_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class assignment(parser.assignment):
    '''
    unique_id = case_generate_alternative : assignment
    '''

    def __init__(self, sString):
        parser.assignment.__init__(self, sString)




from vsg.rules import token_case_n_token_after_tokens_between_tokens

from vsg import token

lTokens = []
lTokens.append(token.interface_constant_declaration.colon)
lTokens.append(token.interface_variable_declaration.colon)
lTokens.append(token.interface_signal_declaration.colon)
lTokens.append(token.interface_unknown_declaration.colon)

oStart = token.generic_clause.open_parenthesis
oEnd = token.generic_clause.close_parenthesis


class rule_017(token_case_n_token_after_tokens_between_tokens):
    '''
    This rule checks the generic type has proper case if it is a VHDL keyword.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

      generic (
        g_width : STD_LOGIC := '0';
        g_depth : Std_logic := '1'
      );

    **Fix**

    .. code-block:: vhdl

      generic (
        g_width : std_logic := '0';
        g_depth : std_logic := '1'
      );
    '''

    def __init__(self):
        token_case_n_token_after_tokens_between_tokens.__init__(self, 'generic', '017', 1, lTokens, oStart, oEnd, True)
        self.disabled = True
        self.groups.append('case::keyword')




from vsg.token import if_generate_statement as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import condition
from vsg.vhdlFile.classify import generate_statement_body


def detect(iToken, lObjects):
    '''
    if_generate_statement ::=
        *generate*_label :
            if [ *alternative*_label : ] condition generate
                generate_statement_body
            { elsif [ *alternative_label* : ] condition generate
                generate_statement_body }
            [ else [ *alternative_label* : ] generate
                generate_statement_body ]
            end generate [ *generate*_label ] ;
    '''

    if utils.are_next_consecutive_tokens([None, ':', 'if'], iToken, lObjects):
        return classify(iToken, lObjects)
    if utils.are_next_consecutive_tokens(['if'], iToken, lObjects):
        iIndex = utils.find_next_token(iToken, lObjects)
        oToken = token.if_keyword(lObjects[iToken].get_value())
        utils.print_error_message('generate_label', oToken, iIndex, lObjects)
    return iToken


def classify(iToken, lObjects):
    iCurrent = utils.tokenize_label(iToken, lObjects, token.generate_label, token.label_colon)

    iCurrent = utils.assign_next_token_required('if', token.if_keyword, iCurrent, lObjects)

    ### Need to handle alternaive_label ###

    iCurrent = condition.classify_until(['generate'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('generate', token.generate_keyword, iCurrent, lObjects)

    iCurrent = generate_statement_body.classify(iCurrent, lObjects)

    while utils.is_next_token('elsif', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('elsif', token.elsif_keyword, iCurrent, lObjects)

        iCurrent = condition.classify_until(['generate'], iCurrent, lObjects)

        iCurrent = utils.assign_next_token_required('generate', token.generate_keyword, iCurrent, lObjects)

        iCurrent = generate_statement_body.classify(iCurrent, lObjects)

    if utils.is_next_token('else', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('else', token.else_keyword, iCurrent, lObjects)

        iCurrent = utils.assign_next_token_required('generate', token.generate_keyword, iCurrent, lObjects)

        iCurrent = generate_statement_body.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('end', token.end_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('generate', token.end_generate_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if_not(';', token.end_generate_label, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import utils

from vsg.rules.whitespace_between_tokens import Rule as WhitespaceRule


class Rule(WhitespaceRule):
    '''
    Checks for a single space after a token

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : token type object list
       A list of tokens to check for a single space after.
    '''

    def __init__(self, name, identifier, lTokens):
        WhitespaceRule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens

    def _get_tokens_of_interest(self, oFile):
        lToi = oFile.get_token_and_n_tokens_after_it(self.lTokens, 2)
        lToi = utils.remove_toi_if_token_is_at_the_end_of_the_line(lToi)
        return lToi




from vsg.vhdlFile.extract import tokens


def get_line_count_between_tokens(oStart, oEnd, lAllTokens, oTokenMap):
    lReturn = []

    lStart, lEnd = oTokenMap.get_token_pair_indexes(oStart, oEnd)

    lStartLines = []
    lDeltaLines = []
    for iStart, iEnd in zip(lStart, lEnd):
        iStartLine = oTokenMap.get_line_number_of_index(iStart)
        lStartLines.append(iStartLine)

        iEndLine = oTokenMap.get_line_number_of_index(iEnd)
        lDeltaLines.append(iEndLine - iStartLine)

    for iStart, iLine, iDelta in zip(lStart, lStartLines, lDeltaLines):
        oTokens = tokens.New(None, iLine, [])
        oTokens.set_token_value(iDelta)
        lReturn.append(oTokens)

    return lReturn




from vsg.rules import blank_line_below_line_ending_with_token

from vsg.token import context_declaration as token


class rule_025(blank_line_below_line_ending_with_token):
    '''
    This rule adds a blank line below the context semicolon.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

       end context;
       entity fifo is

    **Fix**

    .. code-block:: vhdl

       end context;

       entity fifo is
    '''

    def __init__(self):
        blank_line_below_line_ending_with_token.__init__(self, 'context', '025', [token.semicolon])




from vsg.rules.whitespace_between_tokens import Rule

from vsg.token import entity_declaration as token


class rule_013(Rule):
    '''
    This rule checks for a single space after the **entity** keyword in the closing of the entity declaration.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       end entity    fifo;

    **Fix**

    .. code-block:: vhdl

       end entity fifo;
    '''
    def __init__(self):
        Rule.__init__(self, 'entity', '013')
        self.left_token = token.end_entity_keyword
        self.right_token = token.entity_simple_name




from vsg import parser


class generic_keyword(parser.keyword):
    '''
    unique_id = interface_package_generic_map_aspect : generic_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class map_keyword(parser.keyword):
    '''
    unique_id = interface_package_generic_map_aspect : map_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class open_parenthesis(parser.open_parenthesis):
    '''
    unique_id = interface_package_generic_map_aspect : open_parenthesis
    '''

    def __init__(self, sString='('):
        parser.open_parenthesis.__init__(self)


class default_keyword(parser.keyword):
    '''
    unique_id = interface_package_generic_map_aspect : default_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class undefined_range(parser.undefined_range):
    '''
    unique_id = interface_package_generic_map_aspect : undefined_range
    '''

    def __init__(self, sString='<>'):
        parser.undefined_range.__init__(self)


class close_parenthesis(parser.close_parenthesis):
    '''
    unique_id = interface_package_generic_map_aspect : close_parenthesis
    '''

    def __init__(self, sString=')'):
        parser.close_parenthesis.__init__(self)




from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import alignment
from vsg.vhdlFile import utils


class align_left_token_with_right_token_if_right_token_starts_a_line(alignment.Rule):
    '''
    Checks for a single space between two tokens.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : token object list
       List of tokens to align

    left_token : token object
       The first token that defines the region

    right_token : token object
       The second token that defines the region
    '''

    def __init__(self, name, identifier, left_token, right_token):
        alignment.Rule.__init__(self, name=name, identifier=identifier)
        self.left_token = left_token
        self.right_token = right_token
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        lToi = oFile.get_tokens_bounded_by(self.left_token, self.right_token)
        lReturn = []
        for oToi in lToi:
            lTokens = oToi.get_tokens()

            if not utils.does_token_start_line(len(lTokens) - 1, lTokens):
                continue

            oToi.set_meta_data('iRightColumn', oFile.get_column_of_token_index(oToi.get_start_index()))
            oToi.set_meta_data('indentLevel', oFile.get_indent_of_line_at_index(oToi.get_start_index()))
            lReturn.append(oToi)

        return lReturn

    def _analyze(self, lToi):

        for oToi in lToi:

            sActualWhitespace = extract_whitespace_before_last_token(oToi)
            sExpectedWhitespace = expected_whitespace_before_last_token(self, oToi)

            if sActualWhitespace != sExpectedWhitespace:
                oViolation = create_violation(oToi, sActualWhitespace, sExpectedWhitespace)
                self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        if dAction['action'] == 'insert':
            rules_utils.insert_new_whitespace(lTokens, len(lTokens) - 1, dAction['whitespace'])
        else:
            lTokens[-2].set_value(dAction['whitespace'])
        oViolation.set_tokens(lTokens)


def create_violation(oToi, sActualWhitespace, sExpectedWhitespace):
    iLine, lTokens = utils.get_toi_parameters(oToi)
    iLineNumber = iLine + utils.count_token_types_in_list_of_tokens(parser.carriage_return, lTokens)
    sSolution = 'Align ' + lTokens[-1].get_value() + ' with ' + lTokens[0].get_value()
    dAction = {}
    if sActualWhitespace == '':
        dAction['action'] = 'insert'
    else:
        dAction['action'] = 'adjust'
    dAction['whitespace'] = sExpectedWhitespace
    oViolation = violation.New(iLineNumber, oToi, sSolution)
    oViolation.set_action(dAction)
    return oViolation


def extract_whitespace_before_last_token(oToi):
    lTokens = oToi.get_tokens()
    iWhitespaceIndex = len(lTokens) - 2
    if rules_utils.token_is_whitespace(lTokens[iWhitespaceIndex]):
        return lTokens[iWhitespaceIndex].get_value()
    return ''


def expected_whitespace_before_last_token(self, oToi):
    sReturn = indent_whitespace(self, oToi)
    sReturn += aligning_whitespace(self, oToi)
    return sReturn


def indent_whitespace(self, oToi):
    iIndentLevel = oToi.get_meta_data('indentLevel')
    if self.indentStyle == 'smart_tabs':
        return '\t' * iIndentLevel
    return ' ' * iIndentLevel * self.indentSize


def aligning_whitespace(self, oToi):
    iIndentLength = len(indent_whitespace(self, oToi))
    iRightColumn = oToi.get_meta_data('iRightColumn')
    return ' ' * (iRightColumn - iIndentLength)




from vsg import token

from vsg.rules import number_of_lines_between_tokens

oLeftToken = token.process_statement.process_keyword
oRightToken = token.process_statement.semicolon

iLines = 500


class rule_003(number_of_lines_between_tokens):
    '''
    This rule checks the length of a process statement.

    |configuring_length_rules_link|
    '''

    def __init__(self):
        number_of_lines_between_tokens.__init__(self, 'length', '003', oLeftToken, oRightToken, iLines)




from vsg.token import concurrent_assertion_statement as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import assertion


def detect(iToken, lObjects):
    '''
    concurrent_assertion_statement ::=
        [ label : ] [ postponed ] assertion ;

    assertion ::=
        assert condition
            [ report expression ]
            [ severity expression ]

    '''

    if utils.find_in_next_n_tokens('assert', 4, iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.label_name, token.label_colon)

    iCurrent = utils.assign_next_token_if('postponed', token.postponed_keyword, iCurrent, lObjects)

    iCurrent = assertion.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import align_left_token_with_right_token_if_right_token_starts_a_line

from vsg import token

oLeftToken = token.process_statement.open_parenthesis
oRightToken = token.process_statement.close_parenthesis


class rule_028(align_left_token_with_right_token_if_right_token_starts_a_line):
    '''
    This rule checks the alignment of the closing parenthesis of a sensitivity list.
    Parenthesis on multiple lines should be in the same column.

    **Violation**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                           )

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        )
    '''

    def __init__(self):
        align_left_token_with_right_token_if_right_token_starts_a_line.__init__(self, 'process', '028', oLeftToken, oRightToken)




from vsg.rules.whitespace_between_tokens import Rule

from vsg.token import context_declaration as token


class rule_018(Rule):
    '''
    This rule checks for a single space between the **end** keyword and the **context** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       end;

       end   context;

    **Fix**

    .. code-block:: vhdl

       end;

       end context;
    '''
    def __init__(self):
        Rule.__init__(self, 'context', '018')
        self.left_token = token.end_keyword
        self.right_token = token.end_context_keyword




from vsg.token import timeout_clause as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import expression



def detect(iToken, lObjects):
    '''
    timeout_clause ::=
        for *time*_expression
    '''

    if utils.is_next_token('for', iToken, lObjects):
        return True
    return False


def classify_until(lUntils, iToken, lObjects):

    iCurrent = utils.assign_next_token_required('for', token.for_keyword, iToken, lObjects)

    iCurrent = expression.classify_until(lUntils, iCurrent, lObjects)

    return iCurrent




from vsg import parser


class process_label(parser.label):
    '''
    unique_id = process_statement : process_label
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class label_colon(parser.label_colon):
    '''
    unique_id = process_statement : label_colon
    '''

    def __init__(self):
        parser.label_colon.__init__(self)


class postponed_keyword(parser.keyword):
    '''
    unique_id = process_statement : postponed_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class process_keyword(parser.keyword):
    '''
    unique_id = process_statement : process_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class open_parenthesis(parser.open_parenthesis):
    '''
    unique_id = process_statement : open_parenthesis
    '''

    def __init__(self, sString='('):
        parser.open_parenthesis.__init__(self)


class close_parenthesis(parser.close_parenthesis):
    '''
    unique_id = process_statement : close_parenthesis
    '''

    def __init__(self, sString=')'):
        parser.close_parenthesis.__init__(self)


class is_keyword(parser.keyword):
    '''
    unique_id = process_statement : is_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class begin_keyword(parser.keyword):
    '''
    unique_id = process_statement : begin_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_keyword(parser.keyword):
    '''
    unique_id = process_statement : end_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_postponed_keyword(parser.keyword):
    '''
    unique_id = process_statement : end_postponed_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_process_keyword(parser.keyword):
    '''
    unique_id = process_statement : end_process_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_process_label(parser.label):
    '''
    unique_id = process_statement : end_process_label
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = process_statement : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.package_body.end_package_keyword)


class rule_505(token_case):
    '''
    This rule checks the **package** keyword in the **end package body** has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       end PACKAGE body fifo_pkg;

    **Fix**

    .. code-block:: vhdl

       end package body fifo_pkg;
    '''

    def __init__(self):
        token_case.__init__(self, 'package_body', '505', lTokens)
        self.groups.append('case::keyword')




from vsg.rules import move_token_next_to_another_token as Rule

from vsg.token import entity_declaration as token


class rule_023(Rule):
    '''
    This rule checks the end **entity** keyword is on the same line as the **end** keyword.

    **Violation**

    .. code-block:: vhdl

       end entity;

       end
         entity;

    **Fix**

    .. code-block:: vhdl

       end entity;

       end entity;
    '''

    def __init__(self):
        Rule.__init__(self, 'entity', '023', token.end_keyword, token.end_entity_keyword)
        self.subphase = 1
        self.solution = 'Move identifier next to *entity* keyword'




from vsg.rules.whitespace_before_token import Rule

from vsg import token

lTokens = []
lTokens.append(token.selected_expressions.when_keyword)
lTokens.append(token.selected_waveforms.when_keyword)


class rule_106(Rule):
    '''
    This rule checks for a single space before the **when** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       with mux_sel select addr <=
         "0000"    when 0,
         "0001"when 1,
         "1111" when others;

    **Fix**

    .. code-block:: vhdl

       with mux_sel select addr <=
         "0000" when 0,
         "0001" when 1,
         "1111" when others;
    '''

    def __init__(self):
        Rule.__init__(self, 'selected_assignment', '106', lTokens)




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.subtype_declaration.subtype_keyword)


class rule_001(token_indent):
    '''
    This rule checks for indentation of the **subtype** keyword.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of fifo is

            subtype read_size is range 0 to 9;
       subtype write_size is range 0 to 9;

       begin

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is

         subtype read_size is range 0 to 9;
         subtype write_size is range 0 to 9;

       begin

    '''

    def __init__(self):
        token_indent.__init__(self, 'subtype', '001', lTokens)




from vsg.token import identifier as token

from vsg.vhdlFile import utils


def classify(iToken, lObjects, oType=token.identifier):
    '''
    identifier ::=
        basic_identifier | extended_identifier
    '''

    return utils.assign_next_token(oType, iToken, lObjects)




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.loop_statement.loop_label)
lTokens.append(token.iteration_scheme.for_keyword)


class rule_301(token_indent):
    '''
    This rule checks the indentation of the **for** keyword.

    **Violation**

    .. code-block:: vhdl

       fifo_proc : process () is
       begin

       for index in 4 to 23 loop

         end loop;

       end process;

    **Fix**

    .. code-block:: vhdl

       fifo_proc : process () is
       begin

         for index in 4 to 23 loop

         end loop;

       end process;
    '''

    def __init__(self):
        token_indent.__init__(self, 'iteration_scheme', '301', lTokens)




from vsg.token import simple_waveform_assignment as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import delay_mechanism
from vsg.vhdlFile.classify import waveform


def detect(iToken, lObjects):
    '''
    simple_waveform_assignment ::=
        target <= [ delay_mechanism ] waveform ;
    '''

    if utils.is_next_token_one_of(['when', 'if', 'elsif', 'else'], iToken, lObjects):
        return iToken
    if is_a_simple_waveform_assignment(iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_tokens_until('<=', token.target, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('<=', token.assignment, iCurrent, lObjects)

    iCurrent = delay_mechanism.detect(iCurrent, lObjects)

    iCurrent = waveform.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent


def is_a_simple_waveform_assignment(iToken, lObjects):
    if utils.assignment_operator_found(iToken, lObjects):
        if force_or_release_keyword_found(iToken, lObjects):
            return False
        return True
    return False


def force_or_release_keyword_found(iToken, lObjects):
    if utils.find_in_range('force', iToken, ';', lObjects):
        return True
    if utils.find_in_range('release', iToken, ';', lObjects):
        return True
    return False




from vsg import parser


class comma(parser.comma):
    '''
    unique_id = waveform : comma
    '''

    def __init__(self, sString=','):
        parser.comma.__init__(self)


class unaffected_keyword(parser.keyword):
    '''
    unique_id = waveform : unaffected_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg.rules import align_tokens_in_region_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.interface_signal_declaration.colon)
lAlign.append(token.interface_constant_declaration.colon)
lAlign.append(token.interface_variable_declaration.colon)
lAlign.append(token.interface_unknown_declaration.colon)


class rule_410(align_tokens_in_region_between_tokens):
    '''
    This rule checks the alignment of the colon for each parameter in the procedure declaration.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

         procedure average_samples (
           constant a : in integer;
           signal d : out std_logic
         );

    **Fix**

    .. code-block:: vhdl

         procedure average_samples (
           constant a : in integer;
           signal d   : out std_logic
         );
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens.__init__(
            self, 'procedure', '410', lAlign,
            token.procedure_specification.open_parenthesis, token.procedure_specification.close_parenthesis)
        self.solution = 'Align :.'




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.package_declaration.package_keyword, token.package_declaration.identifier])
lTokens.append([token.package_declaration.identifier, token.package_declaration.is_keyword])


class rule_002(Rule):
    '''
    This rule checks for a single space between **package** and **is** keywords.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       package   FIFO_PKG   is

    **Fix**

    .. code-block:: vhdl

       package FIFO_PKG is
    '''
    def __init__(self):
        Rule.__init__(self, 'package', '002', lTokens)





from vsg.rules import move_token_next_to_another_token

from vsg.token import loop_statement as token


class rule_003(move_token_next_to_another_token):
    '''
    This rule checks the **end** keyword is on the same line as the **end loop** keyword.

    **Violation**

    .. code-block:: vhdl

       end
       loop;

    **Fix**

    .. code-block:: vhdl

       end loop;
    '''

    def __init__(self):
        move_token_next_to_another_token.__init__(self, 'loop_statement', '003', token.end_keyword, token.end_loop_keyword)
        self.solution = 'Ensure *end* keyword is on the same line as the *loop* keyword.'




from vsg.rules import token_indent as Rule

from vsg.token import record_type_definition as token


class rule_300(Rule):
    '''
    This rule checks the indent of the **record** keyword if it is on its own line.

    **Violation**

    .. code-block:: vhdl

       type t_record is
            record
         a : std_logic;
         b : std_logic;
       end record t_record;

    **Fix**

    .. code-block:: vhdl

       type t_record is
       record
         a : std_logic;
         b : std_logic;
       end record t_record;
    '''

    def __init__(self):
        Rule.__init__(self, 'record_type_definition', '300', [token.record_keyword])




from vsg.rules import insert_token_right_of_token_if_it_does_not_exist_before_token

from vsg.token import entity_declaration as token


class rule_015(insert_token_right_of_token_if_it_does_not_exist_before_token):
    '''
    This rule checks for the keyword **entity** in the **end entity** statement.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end fifo;

       end;

    **Fix**

    .. code-block:: vhdl

       end entity fifo;

       end entity;
    '''

    def __init__(self):
        insert_token_right_of_token_if_it_does_not_exist_before_token.__init__(self, 'entity', '015', token.end_entity_keyword('entity'), token.end_keyword, token.semicolon)
        self.solution = '*entity* keyword'




from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import block_declarative_item


def detect(iToken, lObjects):
    '''
    architecture_declarative_part ::=
        { block_declarative_item }
    '''

    return utils.detect_submodule(iToken, lObjects, block_declarative_item)




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment as Rule

from vsg import token

lTokens = []
lTokens.append(token.concurrent_selected_signal_assignment.assignment)
lTokens.append(token.selected_force_assignment.assignment)
lTokens.append(token.selected_variable_assignment.assignment)
lTokens.append(token.selected_waveform_assignment.assignment)


class rule_005(Rule):
    '''
    This rule checks for code after the assignment.

    **Violation**

    .. code-block:: vhdl

       with mux_sel select addr <= "0000" when 0,
         "0001" when 1,
         "1111" when others;

    **Fix**

    .. code-block:: vhdl

       with mux_sel select addr <=
         "0000" when 0,
         "0001" when 1,
         "1111" when others;
    '''

    def __init__(self):
        Rule.__init__(self, 'selected_assignment', '005', lTokens)
        self.solution = 'Move code after assignment to the next line.'




from vsg.rules import token_case_in_range_bounded_by_tokens

from vsg import token

lTokens = []
lTokens.append(token.type_mark.name)

oStartToken = token.attribute_declaration.colon

oEndToken = token.attribute_declaration.semicolon


class rule_502(token_case_in_range_bounded_by_tokens):
    '''
    This rule checks the *type_mark* has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       attribute max_delay : TIME;

    **Fix**

    .. code-block:: vhdl

       attribute max_delay : time;
    '''

    def __init__(self):
        token_case_in_range_bounded_by_tokens.__init__(self, 'attribute_declaration', '502', lTokens, oStartToken, oEndToken)
        self.groups.append('case::name')
        self.configuration.append('case_exceptions')




from vsg import token

from vsg.rules import move_token_left_to_next_non_whitespace_token as Rule

lTokens = []
lTokens.append(token.constant_declaration.assignment_operator)


class rule_007(Rule):
    '''
    This rule checks the **:=** is on the same line at the **constant** keyword.

    **Violation**

    .. code-block:: vhdl

       constant size : integer
          := 1;
       constant width : integer
          := 32;

    **Fix**

    .. code-block:: vhdl

       constant size    : integer :=
         1;
       constant width   : integer :=
         32
    '''

    def __init__(self):
        Rule.__init__(self, 'constant', '007', lTokens)
        self.bRemoveTrailingWhitespace = False




from vsg import parser


class New():

    def __init__(self, iStartIndex, iLine, lTokens):

        self.iStartIndex = iStartIndex
        self.lTokens = lTokens
        self.iLine = iLine
        self.sTokenValue = None
        self.iEndIndex = calculate_end_index(iStartIndex, lTokens)
        self.dMetaData = {}

    def get_tokens(self):
        return self.lTokens

    def set_tokens(self, lTokens):
        self.lTokens = lTokens

    def get_line_number(self):
        return self.iLine

    def set_token_value(self, sToken):
        self.sTokenValue = sToken

    def get_token_value(self):
        return self.sTokenValue

    def extract_tokens(self, iStart, iEnd):
        lTokens = self.lTokens[iStart:iEnd + 1]
        iStartIndex = iStart + self.iStartIndex
        iLine = self.iLine
        for iIndex in range(0, iStart):
            try:
                if isinstance(self.lTokens[iIndex], parser.carriage_return):
                    iLine += 1
            except IndexError:
                iLine = iLine
        return New(iStartIndex, iLine, lTokens)

    def get_start_index(self):
        return self.iStartIndex

    def get_end_index(self):
        return self.iEndIndex

    def get_first_token_matching(self, oTokenType):
        for oToken in self.lTokens:
            if isinstance(oToken, oTokenType):
                return oToken
        return None

    def token_type_exists(self, oTokenType):
        for oToken in self.lTokens:
            if isinstance(oToken, oTokenType):
                return True
        return False

    def set_meta_data(self, sKey, sValue):
        self.dMetaData[sKey] = sValue

    def get_meta_data(self, sKey):
        return self.dMetaData[sKey]

    def get_index_of_token_matching(self, oTokenType):
        for iToken, oToken in enumerate(self.lTokens):
            if isinstance(oToken, oTokenType):
                return iToken
        return None


def calculate_end_index(iStartIndex, lTokens):
    try:
        iReturn = iStartIndex
        for oToken in lTokens:
            if not isinstance(oToken, parser.beginning_of_file):
                iReturn += 1
        return iReturn
    except TypeError:
        return None




from vsg import parser
from vsg import token

from vsg.rules import consistent_token_case

lTokens = []
lTokens.append(token.procedure_specification.designator)

lIgnore = []
lIgnore.append(parser.whitespace)
lIgnore.append(parser.carriage_return)
lIgnore.append(parser.blank_line)


class rule_007(consistent_token_case):
    '''
    This rule checks for consistent capitalization of procedure names.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of entity1 is

         procedure average_samples (
           constant a : in integer;
           signal d : out std_logic
         ) is

       begin

         proc1 : process () is
         begin

           Average_samples();

         end process proc1;

       end architecture rtl;

    **Fix**

    .. code-block:: vhdl

       architecture rtl of entity1 is

         procedure average_samples (
           constant a : in integer;
           signal d : out std_logic
         ) is

       begin

         proc1 : process () is
         begin

           average_samples();

         end process proc1;

       end architecture RTL;
    '''

    def __init__(self):
        consistent_token_case.__init__(self, 'procedure', '007', lTokens, lIgnore)




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.context_declaration.context_keyword)


class rule_004(token_case):
    '''
    This rule checks the **context** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       CONTEXT c1 is

    **Fix**

    .. code-block:: vhdl

       context c1 is
    '''

    def __init__(self):
        token_case.__init__(self, 'context', '004', lTokens)
        self.groups.append('case::keyword')




from vsg.token import identifier_list as token

from vsg.vhdlFile import utils


def classify_until(lUntils, iToken, lObjects, oToken=token.identifier):
    '''
    identifier_list ::=
        identifier { , identifier }
    '''
    iEnd = len(lObjects) - 1
    iCurrent = iToken

    while not utils.is_next_token_one_of(lUntils, iCurrent, lObjects):
        if iCurrent == iEnd:
            return iCurrent
        iCurrent = utils.assign_next_token_if_not(',', oToken, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_if(',', token.comma, iCurrent, lObjects)

    return iCurrent




from vsg.token import concurrent_simple_signal_assignment as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import delay_mechanism
from vsg.vhdlFile.classify import waveform


def detect(iToken, lObjects):
    '''
    [ label : ] [ postponed ] concurrent_simple_signal_assignment

    concurrent_simple_signal_assignment ::=
        target <= [ guarded ] [ delay_mechanism ] waveform ;

    The key to detecting this is looking for an assignment <= followed by a semicolon.
    This will be the default if the other types are not found.
    '''

    if not utils.assignment_operator_found(iToken, lObjects):
        return False
    if utils.find_in_range('when', iToken, ';', lObjects):
        return False
    return True


def classify(iToken, lObjects):

    iCurrent = utils.assign_tokens_until('<=', token.target, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('<=', token.assignment, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('guarded', token.guarded_keyword, iCurrent, lObjects)

    iCurrent = delay_mechanism.detect(iCurrent, lObjects)

    iCurrent = waveform.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.token import library_clause as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import logical_name_list


def detect(iToken, lObjects):
    '''
    library_clause ::=
        library logic_name_list ;
    '''
    if utils.is_next_token('library', iToken, lObjects):
        iCurrent = classify(iToken, lObjects)
        return iCurrent
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('library', token.keyword, iToken, lObjects)

    iCurrent = logical_name_list.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_suffix

from vsg import token

lTokens = []
lTokens.append(token.incomplete_type_declaration.identifier)
lTokens.append(token.full_type_declaration.identifier)


class rule_600(token_suffix):
    '''
    This rule checks for valid suffixes in user defined type identifiers.
    The default new type suffix is *\_t*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       type my_type is range -5 to 5 ;

    **Fix**

    .. code-block:: vhdl

       type my_type_t is range -5 to 5 ;
    '''

    def __init__(self):
        token_suffix.__init__(self, 'type', '600', lTokens)
        self.suffixes = ['_t']
        self.solution = 'Type identifiers'




from vsg.rules import move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens

from vsg.token import context_declaration as token

lAnchorTokens = []
lAnchorTokens.append(token.end_keyword)
lAnchorTokens.append(token.end_context_keyword)

oToken = token.context_simple_name

oStartToken = token.end_keyword
oEndToken = token.semicolon


class rule_010(move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens):
    '''
    This rule checks the context identifier is on the same line as the end context keyword.

    **Violation**

    .. code-block:: vhdl

       end context
       c1;

    **Fix**

    .. code-block:: vhdl

       end context c1;
    '''

    def __init__(self):
        move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens.__init__(self, 'context', '010', oToken, lAnchorTokens, oStartToken, oEndToken, bInsertWhitespace=True)
        self.subphase = 2
        self.solution = 'Move context_simple_name next to *context* keyword'




from vsg import parser


class operator(parser.keyword):
    '''
    unique_id = adding_operator : operator
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class plus(operator):
    '''
    unique_id = adding_operator : plus
    '''

    def __init__(self, sString='+'):
        operator.__init__(self, '+')


class minus(operator):
    '''
    unique_id = adding_operator : minus
    '''

    def __init__(self, sString='-'):
        operator.__init__(self, '-')


class concat(operator):
    '''
    unique_id = adding_operator : concat
    '''

    def __init__(self, sString='&'):
        operator.__init__(self, '&')




from vsg.rules import split_line_at_token_if_on_same_line_as_token_if_token_pair_are_not_on_the_same_line

from vsg import token

oToken = token.enumeration_type_definition.close_parenthesis

oSameLineToken = token.enumeration_type_definition.enumeration_literal

lTokenPair = [token.full_type_declaration.type_keyword, token.full_type_declaration.semicolon]


class rule_008(split_line_at_token_if_on_same_line_as_token_if_token_pair_are_not_on_the_same_line):
    '''
    This rule checks the closing parenthesis of multiline enumerated types is on its own line.

    **Violation**

    .. code-block:: vhdl

       type state_machine is (
         idle,
         write,
         read,
         done);

    **Fix**

    .. code-block:: vhdl

       type state_machine is (
         idle,
         write,
         read,
         done
       );
    '''

    def __init__(self):
        split_line_at_token_if_on_same_line_as_token_if_token_pair_are_not_on_the_same_line.__init__(self, 'type', '008', oToken, oSameLineToken, lTokenPair)
        self.solution = "Move enumerated type to the next line."




from vsg import parser
from vsg import token
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import structure
from vsg.vhdlFile import utils


class multiline_structure(structure.Rule):
    '''
    This rule checks the structure of multiline constants that contain arrays.

    |configuring_multiline_structure_rules_link|

    .. NOTE:: The indenting of multiline array constants is handled by the rule `constant_012 <constant_rules.html#constant-012>`_.

    **Violation**

    .. code-block:: vhdl

       constant rom : romq_type := (0, 65535, 32768);

    **Fix**

    .. code-block:: vhdl

       constant rom : romq_type :=
       (
         0,
         65535,
         32768
       );
    '''

    def __init__(self, name, identifier, lTokenPairs):
        structure.Rule.__init__(self, name=name, identifier=identifier)
        self.phase = 1
        self.lTokenPairs = lTokenPairs
        self.bExcludeLastToken = True
        self.assignment_operator = None
        self.configuration_documentation_link = 'configuring_array_multiline_structure_rules_link'

        self.first_paren_new_line = 'yes'
        self.configuration.append('first_paren_new_line')
        self.last_paren_new_line = 'yes'
        self.configuration.append('last_paren_new_line')
        self.open_paren_new_line = 'yes'
        self.configuration.append('open_paren_new_line')
        self.close_paren_new_line = 'yes'
        self.configuration.append('close_paren_new_line')
        self.new_line_after_comma = 'yes'
        self.configuration.append('new_line_after_comma')
        self.assign_on_single_line = 'yes'
        self.configuration.append('assign_on_single_line')
        self.ignore_single_line = 'yes'
        self.configuration.append('ignore_single_line')
        self.move_last_comment = 'ignore'

    def _get_tokens_of_interest(self, oFile):
        lToi = []
        for lTokenPair in self.lTokenPairs:
            aToi = oFile.get_tokens_bounded_by(lTokenPair[0], lTokenPair[1], bExcludeLastToken=self.bExcludeLastToken, bIncludeTillEndOfLine=True)
            lToi = utils.combine_two_token_class_lists(lToi, aToi)

        lReturn = []
        for oToi in lToi:
            if rules_utils.is_single_line(oToi) and self.ignore_single_line == 'yes':
                continue

#            if not _is_open_paren_after_assignment(self, oToi):
#                continue

            lReturn.append(oToi)

        lMyReturn = remove_non_arrays(self.assignment_operator, lReturn)

        return lMyReturn

    def _analyze(self, lToi):
        for oToi in lToi:

            _check_first_paren_new_line(self, oToi)
            _check_last_paren_new_line(self, oToi)
            _check_open_paren_new_line(self, oToi)
            _check_close_paren_new_line(self, oToi)
            _check_new_line_after_comma(self, oToi)
            _check_assign_on_single_line(self, oToi)

        self._sort_violations()

    def _fix_violation(self, oViolation):
        dAction = oViolation.get_action()
        dAction['type'](oViolation)


def _check_first_paren_new_line(self, oToi):

    if self.first_paren_new_line == 'ignore':
        return

    iLine, lTokens = utils.get_toi_parameters(oToi)
    bSearch = False
    for iToken, oToken in enumerate(lTokens):
        iLine = utils.increment_line_number(iLine, oToken)
        if isinstance(oToken, self.assignment_operator):
            iStart = iToken
            bSearch = True
        if isinstance(oToken, parser.open_parenthesis) and bSearch:
            if utils.find_carriage_return(lTokens[iStart:iToken]) is None:
                if self.first_paren_new_line == 'yes':
                    sSolution = 'Move parenthesis after assignment to the next line.'
                    oToi.set_meta_data('iLine', iLine)
                    oToi.set_meta_data('iStartIndex', iToken - 1)
                    oToi.set_meta_data('iEndIndex', iToken)
                    oToi.set_meta_data('sType', _fix_first_paren_new_line)
                    oToi.set_meta_data('sAction', 'insert')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)
            else:
                if self.first_paren_new_line == 'no':
                    sSolution = 'Move parenthesis to same line as assignment operator.'
                    oToi.set_meta_data('iLine', iLine)
                    oToi.set_meta_data('iStartIndex', iStart)
                    oToi.set_meta_data('iEndIndex', iToken)
                    oToi.set_meta_data('sType', _fix_first_paren_new_line)
                    oToi.set_meta_data('sAction', 'remove')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)
            break


def _check_last_paren_new_line(self, oToi):

    if self.last_paren_new_line == 'ignore':
        return
    iLine, lTokens = utils.get_toi_parameters(oToi)
    lTokens.reverse()
    iLine = iLine + utils.count_carriage_returns(lTokens)
    bReturnFound = False
    bCommentFound = False
    for iToken, oToken in enumerate(lTokens):
        iLine = utils.decrement_line_number(iLine, oToken)
        if isinstance(oToken, parser.comment):
            bCommentFound = True
        if isinstance(oToken, parser.close_parenthesis):
            iEnd = len(lTokens) - iToken - 1
            if utils.are_next_consecutive_token_types([parser.whitespace, parser.carriage_return], iToken + 1, lTokens):
                bReturnFound = True
            elif utils.are_next_consecutive_token_types([parser.carriage_return], iToken + 1, lTokens):
                bReturnFound = True

            lTokens.reverse()

            if self.last_paren_new_line == 'yes' and not bReturnFound:
                if self.move_last_comment == 'yes' and bCommentFound:
                    sSolution = 'Move parenthesis after assignment to the next line and trailing comment to previous line.'
                    oToi.set_meta_data('iLine', iLine)
                    oToi.set_meta_data('iStartIndex', iEnd - 1)
                    oToi.set_meta_data('iEndIndex', len(lTokens) - 1)
                    oToi.set_meta_data('sType', _fix_last_paren_new_line)
                    oToi.set_meta_data('sAction', 'insert_and_move_comment')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    oViolation.semicolon = self.semicolon
                    self.add_violation(oViolation)
                else:
                    sSolution = 'Move closing parenthesis to the next line.'
                    oToi.set_meta_data('iLine', iLine)
                    oToi.set_meta_data('iStartIndex', iEnd - 1)
                    oToi.set_meta_data('iEndIndex', iEnd)
                    oToi.set_meta_data('sType', _fix_last_paren_new_line)
                    oToi.set_meta_data('sAction', 'insert')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    oViolation.semicolon = self.semicolon
                    self.add_violation(oViolation)
            elif self.last_paren_new_line == 'no' and bReturnFound:
                iStart = utils.find_previous_non_whitespace_token(iEnd - 1, lTokens)
                oToi.set_meta_data('iLine', iLine)
                oToi.set_meta_data('iStartIndex', iStart)
                oToi.set_meta_data('iEndIndex', iEnd)
                oToi.set_meta_data('sType', _fix_last_paren_new_line)
                oToi.set_meta_data('sAction', 'remove')
                oToi.set_meta_data('sSolution', 'Move closing parenthesis to previous line.')
                oViolation = _create_violation(oToi)
                oViolation.semicolon = self.semicolon
                self.add_violation(oViolation)

            break


def _check_open_paren_new_line(self, oToi):

    if self.open_paren_new_line == 'ignore':
        return

    iLine, lTokens = utils.get_toi_parameters(oToi)

    bAssignmentFound = False
    bOthersClause = False

    iToken = _find_assignment_operator(self, lTokens) + 1
    iStopIndex = len(lTokens)
    bFirstParenFound = False
    while iToken < iStopIndex:

        if bFirstParenFound:
            iToken, bOthersClause = _classify_others(iToken, lTokens)

            if bOthersClause:
                iToken += 1
                continue

            iToken, bAssignmentFound = _classify_assignment(iToken, lTokens)

            if bAssignmentFound:
                iToken += 1
                continue

        oToken = lTokens[iToken]

        if isinstance(oToken, parser.open_parenthesis):
            bFirstParenFound = True
            if utils.is_token_at_end_of_line(iToken, lTokens):
                if self.open_paren_new_line == 'no':
                    iEnd = utils.find_next_non_whitespace_token(iToken + 1, lTokens)
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iToken]) + iLine
                    sSolution = 'Remove carriage return after open parenthesis.'
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iToken)
                    oToi.set_meta_data('iEndIndex', iEnd)
                    oToi.set_meta_data('sType', _fix_open_paren_new_line)
                    oToi.set_meta_data('sAction', 'remove')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)
            else:
                if self.open_paren_new_line == 'yes':
                    sSolution = 'Add carriage return after open parenthesis.'
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iToken]) + iLine
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iToken)
                    oToi.set_meta_data('iEndIndex', iToken)
                    oToi.set_meta_data('sType', _fix_open_paren_new_line)
                    oToi.set_meta_data('sAction', 'insert')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)
        bOthersClause = False
        iToken += 1

    return None


def _check_close_paren_new_line(self, oToi):

    if self.close_paren_new_line == 'ignore':
        return

    iLine, lTokens = utils.get_toi_parameters(oToi)

    bAssignmentFound = False
    bOthersClause = False

    iToken = _find_assignment_operator(self, lTokens) + 1
    iStopIndex = _find_last_closing_paren(lTokens)
    bFirstParenFound = False
    while iToken < iStopIndex:

        if bFirstParenFound:
            iToken, bOthersClause = _classify_others(iToken, lTokens)

            if bOthersClause:
                iToken += 1
                continue

            iToken, bAssignmentFound = _classify_assignment(iToken, lTokens)

            if bAssignmentFound:
                iToken += 1
                continue

        oToken = lTokens[iToken]

        if isinstance(oToken, parser.open_parenthesis):
            bFirstParenFound = True

        if isinstance(oToken, parser.close_parenthesis):
            if utils.does_token_start_line(iToken, lTokens):
                if self.close_paren_new_line == 'no':
                    iStart = utils.find_previous_non_whitespace_token(iToken - 1, lTokens)
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iToken]) + iLine
                    sSolution = 'Move closing parenthesis to previous line.'
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iStart)
                    oToi.set_meta_data('iEndIndex', iToken)
                    oToi.set_meta_data('sType', _fix_close_paren_new_line)
                    oToi.set_meta_data('sAction', 'remove')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)
            else:
                if self.close_paren_new_line == 'yes':
                    iStart = iToken - 1
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iToken]) + iLine
                    sSolution = 'Move closing parenthesis to the next line.'
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iStart)
                    oToi.set_meta_data('iEndIndex', iToken)
                    oToi.set_meta_data('sType', _fix_close_paren_new_line)
                    oToi.set_meta_data('sAction', 'insert')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)

        iToken += 1

    return None


def _check_new_line_after_comma(self, oToi):

    if self.new_line_after_comma == 'ignore':
        return

    iLine, lTokens = utils.get_toi_parameters(oToi)

    bAssignmentFound = False
    bOthersClause = False
    bPositionalFound = True

    iToken = _find_assignment_operator(self, lTokens) + 1
    iStopIndex = len(lTokens)
    bFirstParenFound = False
    while iToken < iStopIndex:

        if bFirstParenFound:
            iToken, bOthersClause = _classify_others(iToken, lTokens)
            iToken, bAssignmentFound = _classify_assignment(iToken, lTokens)

            if bOthersClause:
                bPositionalFound = True
                iToken += 1

            if bAssignmentFound:
                iToken += 1
                bPositionalFound = False

        oToken = lTokens[iToken]

        if isinstance(oToken, parser.open_parenthesis):
            bFirstParenFound = True

        if isinstance(oToken, parser.comma):
            if bPositionalFound and self.new_line_after_comma == 'ignore_positional':
                iToken += 1
                continue

            if utils.is_token_at_end_of_line(iToken, lTokens):
                if self.new_line_after_comma == 'no':
                    iEnd = utils.find_next_non_whitespace_token(iToken + 1, lTokens)
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iToken]) + iLine
                    sSolution = 'Remove carriage return after comma.'
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iToken)
                    oToi.set_meta_data('iEndIndex', iEnd)
                    oToi.set_meta_data('sType', _fix_new_line_after_comma)
                    oToi.set_meta_data('sAction', 'remove')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)
            else:
                if self.new_line_after_comma == 'yes' or self.new_line_after_comma == 'ignore_positional':
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iToken]) + iLine
                    sSolution = 'Add carriage return after comma.'
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iToken)
                    oToi.set_meta_data('iEndIndex', iToken + 1)
                    oToi.set_meta_data('sType', _fix_new_line_after_comma)
                    oToi.set_meta_data('sAction', 'insert')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)

        iToken += 1

    return None


def _check_assign_on_single_line(self, oToi):

    if self.assign_on_single_line == 'ignore':
        return

    iLine, lTokens = utils.get_toi_parameters(oToi)

    bAssignmentFound = False
    bOthersClause = False

    iToken = _find_assignment_operator(self, lTokens) + 1
    iStopIndex = len(lTokens)
    bFirstParenFound = False
    while iToken < iStopIndex:

        if bFirstParenFound:
            iToken, bOthersClause = _classify_others(iToken, lTokens)

            if bOthersClause:
                iToken += 1
                continue

            if lTokens[iToken].get_value() == '=>':
                iPreviousToken = utils.find_previous_non_whitespace_token(iToken - 1, lTokens)
            iToken, bAssignmentFound = _classify_assignment(iToken, lTokens)

            if bAssignmentFound:
                if rules_utils.number_of_carriage_returns(lTokens[iPreviousToken:iToken]) > 0:
                    iErrorLine = rules_utils.number_of_carriage_returns(lTokens[:iPreviousToken]) + iLine
                    sSolution = 'Remove carriage returns for assignments.'
                    oToi.set_meta_data('iLine', iErrorLine)
                    oToi.set_meta_data('iStartIndex', iPreviousToken)
                    oToi.set_meta_data('iEndIndex', iToken)
                    oToi.set_meta_data('sType', _fix_assign_on_single_line)
                    oToi.set_meta_data('sAction', 'remove')
                    oToi.set_meta_data('sSolution', sSolution)
                    oViolation = _create_violation(oToi)
                    self.add_violation(oViolation)

        oToken = lTokens[iToken]

        if isinstance(oToken, parser.open_parenthesis):
            bFirstParenFound = True

        iToken += 1

    return None


def _is_open_paren_after_assignment(self, oToi):

    lTokens = oToi.get_tokens()
    for iToken, oToken in enumerate(lTokens):
        if isinstance(oToken, self.assignment_operator):
            lExpectedTokens = [self.assignment_operator, parser.open_parenthesis]
            if utils.are_next_consecutive_token_types_ignoring_whitespace(lExpectedTokens, iToken, lTokens):
                return True
    return False


def _fix_first_paren_new_line(oViolation):
    lTokens = oViolation.get_tokens()
    dAction = oViolation.get_action()
    if dAction['action'] == 'insert':
        if not isinstance(lTokens[0], parser.whitespace):
            rules_utils.insert_whitespace(lTokens, 0)
        rules_utils.insert_carriage_return(lTokens, 0)
        oViolation.set_tokens(lTokens)
    elif dAction['action'] == 'remove':
        lNewTokens = []
        lNewTokens.append(lTokens[0])
        rules_utils.append_whitespace(lNewTokens)
        lNewTokens.append(lTokens[-1])
        oViolation.set_tokens(lNewTokens)


def _fix_last_paren_new_line(oViolation):
    lTokens = oViolation.get_tokens()
    dAction = oViolation.get_action()
    if dAction['action'] == 'insert':
        if not isinstance(lTokens[0], parser.whitespace):
            rules_utils.insert_whitespace(lTokens, 1)
        rules_utils.insert_carriage_return(lTokens, 1)
        oViolation.set_tokens(lTokens)
    elif dAction['action'] == 'remove':
        lNewTokens = []
        lNewTokens.append(lTokens[0])
        lNewTokens.append(lTokens[-1])
        oViolation.set_tokens(lNewTokens)
    elif dAction['action'] == 'insert_and_move_comment':
        if not isinstance(lTokens[0], parser.whitespace):
            rules_utils.insert_whitespace(lTokens, 1)
        oSemicolon = oViolation
        iSwapIndex = rules_utils.get_index_of_token_in_list(oViolation.semicolon, lTokens) + 1
        lNewTokens = []
        lNewTokens.append(lTokens[0])
        lNewTokens.extend(lTokens[iSwapIndex:])
        rules_utils.append_carriage_return(lNewTokens)
        lNewTokens.extend(lTokens[1:iSwapIndex])

        oViolation.set_tokens(lNewTokens)


def _fix_open_paren_new_line(oViolation):
    lTokens = oViolation.get_tokens()
    dAction = oViolation.get_action()
    if dAction['action'] == 'insert':
        rules_utils.append_carriage_return(lTokens)
        rules_utils.append_whitespace(lTokens)
        oViolation.set_tokens(lTokens)
    elif dAction['action'] == 'remove':
        lNewTokens = []
        lNewTokens.append(lTokens[0])
        lNewTokens.append(lTokens[-1])
        oViolation.set_tokens(lNewTokens)


def _fix_close_paren_new_line(oViolation):
    lTokens = oViolation.get_tokens()
    dAction = oViolation.get_action()
    if dAction['action'] == 'insert':
        if not isinstance(lTokens[0], parser.whitespace):
            rules_utils.insert_whitespace(lTokens, 1)
        rules_utils.insert_carriage_return(lTokens, 1)
        oViolation.set_tokens(lTokens)
    elif dAction['action'] == 'remove':
        lNewTokens = []
        lNewTokens.append(lTokens[0])
        lNewTokens.append(lTokens[-1])
        oViolation.set_tokens(lNewTokens)


def _fix_new_line_after_comma(oViolation):
    lTokens = oViolation.get_tokens()
    dAction = oViolation.get_action()
    if dAction['action'] == 'insert':
        if isinstance(lTokens[1], parser.whitespace):
            rules_utils.insert_carriage_return(lTokens, 1)
        else:
            rules_utils.insert_whitespace(lTokens, 1)
            rules_utils.insert_carriage_return(lTokens, 1)
        oViolation.set_tokens(lTokens)
    elif dAction['action'] == 'remove':
        lNewTokens = []
        lNewTokens.append(lTokens[0])
        lNewTokens.append(parser.whitespace(' '))
        lNewTokens.append(lTokens[-1])
        oViolation.set_tokens(lNewTokens)


def _fix_assign_on_single_line(oViolation):
    lTokens = oViolation.get_tokens()
    dAction = oViolation.get_action()
    lNewTokens = []
    if dAction['action'] == 'remove':
        lNewTokens = utils.remove_carriage_returns_from_token_list(lTokens)
        lNewTokens = utils.remove_comments_from_token_list(lNewTokens)
        lNewTokens = utils.remove_consecutive_whitespace_tokens(lNewTokens)

        oViolation.set_tokens(lNewTokens)


def _inside_others_clause(iToken, lTokens):
    for oToken in lTokens[iToken + 1:]:
        if utils.token_is_whitespace_or_comment(oToken):
            continue
        elif rules_utils.token_is_parenthesis(oToken):
            return False
        else:
            if oToken.get_value().lower() == 'others':
                break
    else:
        return False
    return True


def _create_violation(oToi):
    iLine = oToi.get_meta_data('iLine')
    iStartIndex = oToi.get_meta_data('iStartIndex')
    iEndIndex = oToi.get_meta_data('iEndIndex')
    sType = oToi.get_meta_data('sType')
    sAction = oToi.get_meta_data('sAction')
    sSolution = oToi.get_meta_data('sSolution')
    dAction = _create_action_dictionary(sType, sAction)
    oViolation = violation.New(iLine, oToi.extract_tokens(iStartIndex, iEndIndex), sSolution)
    oViolation.set_action(dAction)
    return oViolation


def _create_action_dictionary(sType, sAction):
    dReturn = {}
    dReturn['type'] = sType
    dReturn['action'] = sAction
    return dReturn


def _found_assignment_operator(oToken, bSearch):
    if bSearch:
        return True
    if isinstance(oToken, self.assignment_operator):
        return True
    return False


def _find_assignment_operator(self, lTokens):
    for iToken, oToken in enumerate(lTokens):
        if isinstance(oToken, self.assignment_operator):
            return iToken
    return None


def _find_last_closing_paren(lTokens):
    iReturn = None
    for iToken, oToken in enumerate(lTokens):
        if isinstance(oToken, parser.close_parenthesis):
            iReturn = iToken
    return iReturn


def _classify_assignment(iToken, lTokens):

    oToken = lTokens[iToken]
    if oToken.get_value() == '=>':
        iOpenParen = 0
        iCloseParen = 0
        for iReturn, oToken in enumerate(lTokens[iToken:]):
            iOpenParen = rules_utils.update_open_paren_counter(oToken, iOpenParen)
            iCloseParen = rules_utils.update_close_paren_counter(oToken, iCloseParen)
            iEnd = iToken + iReturn
            if something(oToken, iOpenParen, iCloseParen):
                iEnd = utils.find_previous_non_whitespace_token(iEnd - 1, lTokens)
                return iEnd, True
            if something_else(oToken, iOpenParen, iCloseParen):
                return iEnd, True
            if other_thing(oToken, iOpenParen, iCloseParen):
                iEnd = iToken + iReturn - 1
                return iEnd, True
        return None
    return iToken, False


def other_thing(oToken, iOpenParen, iCloseParen):
    if isinstance(oToken, parser.comma) and iOpenParen == iCloseParen:
        return True
    return False


def something_else(oToken, iOpenParen, iCloseParen):
    if isinstance(oToken, parser.close_parenthesis) and iCloseParen == iOpenParen:
            return True
    return False


def something(oToken, iOpenParen, iCloseParen):
    if isinstance(oToken, parser.close_parenthesis) and iCloseParen > iOpenParen:
            return True
    return False


def _classify_others(iToken, lTokens):
    oToken = lTokens[iToken]
    if oToken.get_value() == '(':
        iOpenParen = 0
        iCloseParen = 0
        iStart = iToken
        for iReturn, oToken in enumerate(lTokens[iToken:]):
            if isinstance(oToken, parser.open_parenthesis):
                iOpenParen += 1
            elif isinstance(oToken, parser.close_parenthesis):
                iCloseParen += 1
                if iOpenParen == iCloseParen:
                    break
        else:
            return iToken, False

        iEnd = iReturn + iStart

        for iIndex in range(iStart + 1, iEnd):
            if lTokens[iIndex].get_value().lower() == 'others':
                return iEnd, True

    return iToken, False


def remove_non_arrays(assignment_operator, lToi):
    lReturn = []
    for oToi in lToi:
        if rules_utils.array_detected_after_assignment_operator(assignment_operator, oToi):
            lReturn.append(oToi)
    return lReturn




from vsg.token import interface_file_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import identifier_list
from vsg.vhdlFile.classify import subtype_indication


def detect(iToken, lObjects):
    '''
    interface_file_declaration ::=
        file identifier_list : subtype_indication
    '''

    if utils.is_next_token('file', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('file', token.file_keyword, iToken, lObjects)

    iCurrent = identifier_list.classify_until([':'], iCurrent, lObjects, token.identifier)

    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent = subtype_indication.classify(iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.case_statement_alternative.when_keyword)


class rule_016(token_case):
    '''
    This rule checks the **when** has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

         WHEN a =>
         When b =>
         when c =>

    **Fix**

    .. code-block:: vhdl

         when a =>
         when b =>
         when c =>
    '''

    def __init__(self):
        token_case.__init__(self, 'case', '016', lTokens)
        self.groups.append('case::keyword')



from vsg import parser


class name(parser.name):
    '''
    unique_id = type_mark : name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)


class tic(parser.tic):
    '''
    unique_id = type_mark : tic
    '''

    def __init__(self, sString):
        parser.tic.__init__(self, sString)


class attribute(parser.attribute):
    '''
    unique_id = type_mark : attribute
    '''

    def __init__(self, sString):
        parser.attribute.__init__(self, sString)





from vsg.rules import token_does_not_exist_before_token as Rule
from vsg.token import loop_statement as token

oFirstToken = token.loop_label

oSecondToken = token.loop_keyword


class rule_006(Rule):
    '''
    This rule checks that loop statements have a label.

    **Violation**

    .. code-block:: vhdl

       loop

       while (condition) loop

       for x in range (31 downto 0) loop

    **Fix**

    .. code-block:: vhdl

       LOOP_LABEL : loop

       LOOP_LABEL : while (condition) loop

       LOOP_LABEL : for x in range (31 downto 0) loop
    '''

    def __init__(self):
        Rule.__init__(self, 'loop_statement', '006', oFirstToken, oSecondToken)
        self.solution = 'Add label for loop statement'
        self.fixable = False
        self.disable = True




from vsg import parser


class label(parser.label):
    '''
    unique_id = variable_assignment_statement : label
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class label_colon(parser.colon):
    '''
    unique_id = variable_assignment_statement : label_colon
    '''

    def __init__(self, sString=';'):
        parser.colon.__init__(self, sString)




from vsg.rules import insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token

from vsg.token import package_declaration as token


class rule_014(insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token):
    '''
    This rule checks the package name exists on the same line as the **end package** keywords.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end package;

    **Fix**

    .. code-block:: vhdl

       end package fifo_pkg;
    '''

    def __init__(self):
        insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token.__init__(self, 'package', '014', token.end_package_simple_name, token.semicolon, token.end_keyword, token.semicolon, token.identifier)
        self.solution = 'package name.'




from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import blank_line


class blank_line_below_line_ending_with_token(blank_line.Rule):
    '''
    Checks for a blank line below a line ending with a given token

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    token: token object type list
       token object that a blank line below should appear
    '''

    def __init__(self, name, identifier, lTokens, lAllowTokens=None):
        blank_line.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.lHierarchyLimits = None
        if lAllowTokens is None:
            self.lAllowTokens = []
        else:
            self.lAllowTokens = lAllowTokens
        self.style = 'require_blank_line'
        self.configuration.append('style')

    def _get_tokens_of_interest(self, oFile):
        if self.style == 'require_blank_line':
            if self.lHierarchyLimits is None:
                return oFile.get_line_below_line_ending_with_token(self.lTokens)
            else:
                return oFile.get_line_below_line_ending_with_token_with_hierarchy(self.lTokens, self.lHierarchyLimits)
        elif self.style == 'no_blank_line':
            return oFile.get_blank_lines_below_line_ending_with_token(self.lTokens, self.lHierarchyLimits)

    def _analyze(self, lToi):
        if self.style == 'require_blank_line':
            _analyze_require_blank_line(self, lToi, self.lAllowTokens)
        elif self.style == 'no_blank_line':
            _analyze_no_blank_line(self, lToi, self.lAllowTokens)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        if dAction['action'] == 'Insert':
            rules_utils.insert_carriage_return(lTokens, 0)
            rules_utils.insert_blank_line(lTokens, 0)
            oViolation.set_tokens(lTokens)
        elif dAction['action'] == 'Remove':
            oViolation.set_tokens([])


def _analyze_require_blank_line(self, lToi, lAllowTokens):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            if _is_allowed_token(lAllowTokens, lTokens):
                continue
            if len(lTokens) == 1:
                if isinstance(lTokens[0], parser.blank_line):
                    continue
            sSolution = 'Insert blank line below'
            oViolation = violation.New(oToi.get_line_number() - 1, oToi, sSolution)
            dAction = {}
            dAction['action'] = 'Insert'
            oViolation.set_action(dAction)
            self.add_violation(oViolation)


def _analyze_no_blank_line(self, lToi, lAllowTokens):
        for oToi in lToi:
            sSolution = 'Remove blank lines below'
            oViolation = violation.New(oToi.get_line_number() + 1, oToi, sSolution)
            dAction = {}
            dAction['action'] = 'Remove'
            oViolation.set_action(dAction)
            self.add_violation(oViolation)


def _is_allowed_token(lAllowTokens, lTokens):
    bSkip = False
    for oAllowToken in lAllowTokens:
        for oToken in lTokens:
            if isinstance(oToken, oAllowToken):
                bSkip = True
                break
        if bSkip:
           break
    if bSkip:
        return True
    return False




from .rule_001 import rule_001
from .rule_002 import rule_002
from .rule_003 import rule_003
from .rule_004 import rule_004
from .rule_005 import rule_005
from .rule_006 import rule_006
from .rule_007 import rule_007
from .rule_008 import rule_008
from .rule_009 import rule_009
from .rule_010 import rule_010
from .rule_011 import rule_011
from .rule_012 import rule_012

from .rule_400 import rule_400
from .rule_401 import rule_401




from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens as Rule

from vsg import token

lAlign = []
lAlign.append(token.entity_specification.colon)

oStart = token.process_statement.process_keyword
oEnd = token.process_statement.begin_keyword

lUnless = []
lUnless.append([token.subprogram_body.is_keyword, token.subprogram_body.begin_keyword])


class rule_401(Rule):
    '''
    This rule checks the colons are in the same column for all attribute specifications.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

         attribute mark_debug of wr_en : signal is "true";
         attribute mark_debug of almost_empty : signal is "true";
         attribute mark_debug of full : signal is "true";

    **Fix**

    .. code-block:: vhdl

         attribute mark_debug of wr_en        : signal is "true";
         attribute mark_debug of almost_empty : signal is "true";
         attribute mark_debug of full         : signal is "true";
    '''

    def __init__(self):
        Rule.__init__(self, 'process', '401', lAlign, oStart, oEnd, lUnless)
        self.solution = 'align colon.'




from vsg.token import report_statement

from vsg.rules import align_consecutive_lines_after_line_starting_with_token_and_stopping_with_token as Rule


class rule_400(Rule):
    '''
    This rule checks the alignment of the report expressions.

    .. NOTE:: There is a configuration option **alignment** which changes the indent location of multiple lines.

    alignment set to 'report' (Default)
    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    **Violation**

    .. code-block:: vhdl

       report "FIFO width is limited" &
       " to 16 bits."
         severity FAILURE;

    **Fix**

    .. code-block:: vhdl

       report "FIFO width is limited" &
              " to 16 bits."
         severity FAILURE;

    alignment set to 'left'
    ^^^^^^^^^^^^^^^^^^^^^^^

    **Violation**

    .. code-block:: vhdl

       report "FIFO width is limited" &
       " to 16 bits."
         severity FAILURE;

    **Fix**

    .. code-block:: vhdl

         report "FIFO width is limited" &
             " to 16 bits."
           severity FAILURE;
    '''

    def __init__(self):
        Rule.__init__(self, name="report_statement", identifier="400")
        self.lStartTokens = [report_statement.report_keyword]
        self.lEndTokens = [report_statement.severity_keyword, report_statement.semicolon]
        self.indentAdjust = 2
        self.configuration_documentation_link = None




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.loop_statement.loop_label)
lTokens.append(token.iteration_scheme.while_keyword)


class rule_300(token_indent):
    '''
    This rule checks for indentation of the **while** keyword.
    Proper indentation enhances comprehension.

    **Violation**

    .. code-block:: vhdl

       begin

       while (temp /= 0) loop
           temp := temp/2;
         end loop;

    **Fix**

    .. code-block:: vhdl

       begin

         while (temp /= 0) loop
           temp := temp/2;
         end loop;
    '''

    def __init__(self):
        token_indent.__init__(self, 'iteration_scheme', '300', lTokens)




from vsg.rules import multiline_constraint_structure as Rule

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.constant_declaration.constant_keyword, token.constant_declaration.semicolon])


class rule_017(Rule):
    '''
    This rule checks the structure of constant constraints.

    |configuring_multiline_constraint_rules_link|

    **Violation**

    .. code-block:: vhdl

       constant con_a : my_record(element1(7 downto 0), element2(3 downto 0));

    **Fix**

    .. code-block:: vhdl

       constant con_a : my_record(
           element1(7 downto 0),
           element2(3 downto 0)
         );
    '''

    def __init__(self):
        Rule.__init__(self, 'constant', '017')
        self.lTokenPairs = lTokenPairs




from vsg.rules import token_case as Rule

from vsg import token

lTokens = []
lTokens.append(token.context_reference.context_name)


class rule_501(Rule):
    '''
    This rule checks the context name called out in the selected name has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       context my_lib.INTERFACES;

    **Fix**

    .. code-block:: vhdl

       context my_lib.interfaces;
    '''

    def __init__(self):
        Rule.__init__(self, 'context_ref', '501', lTokens)
        self.groups.append('case::name')
        self.configuration.append('case_exceptions')
